/*
 * $Id: libgnac-converter.h 369 2009-07-14 06:59:02Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifndef __LIBGNAC_CONVERTER_H__
#define __LIBGNAC_CONVERTER_H__

#include <gio/gio.h>
#include <glib.h>
#include <glib-object.h>
#include <gst/gst.h>
#include <gst/pbutils/pbutils.h>

#include "libgnac-metadata.h"
#include "libgnac-metadata-tags.h"

#define PROGRESS_TIMEOUT 250

G_BEGIN_DECLS

#define LIBGNAC_TYPE_CONVERTER (libgnac_converter_get_type())
#define LIBGNAC_CONVERTER(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), \
    LIBGNAC_TYPE_CONVERTER, LibgnacConverter))
#define LIBGNAC_IS_CONVERTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
    LIBGNAC_TYPE_CONVERTER))
#define LIBGNAC_CONVERTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), \
    LIBGNAC_TYPE_CONVERTER, LibgnacConverterClass))
#define LIBGNAC_IS_CONVERTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), \
    LIBGNAC_TYPE_CONVERTER))
#define LIBGNAC_CONVERTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), \
    LIBGNAC_TYPE_CONVERTER, LibgnacConverterClass))

typedef struct LibgnacConverterPrivate LibgnacConverterPrivate;

typedef struct
{
  GObject parent_instance;

  /* private */
  LibgnacConverterPrivate *priv;

} LibgnacConverter;

typedef struct
{
  GObjectClass parent_class;

  /* class members */

  /* signals */
  gboolean (*overwrite) (LibgnacConverter *converter, gpointer file);
  void (*progress) (LibgnacConverter *converter, gfloat fraction);
  void (*file_added) (LibgnacConverter *converter, gchar *uri);
  void (*file_removed) (LibgnacConverter *converter, gchar *uri);
  void (*file_started) (LibgnacConverter *converter, gchar *input);
  void (*file_completed) (LibgnacConverter *converter, gchar *uri);
  void (*files_cleared) (LibgnacConverter *converter);
  void (*started) (LibgnacConverter *converter);
  void (*stopped) (LibgnacConverter *converter);
  void (*completion) (LibgnacConverter *converter);
  void (*error) (LibgnacConverter *converter, gchar *uri, gchar *msg, GError *error);

} LibgnacConverterClass;

/* Encoder item that handle a file conversion */
typedef struct 
{
  LibgnacConverter   *converter;
  GFile              *source;
  GFile              *destination;
  GstElement         *pipeline;
  GstElement         *audio_encoder;
  GstBus             *bus;
  guint               timeout_id;
  gint64              pos;
} LibgnacConverterItem;

GType 
libgnac_converter_get_type(void);

GObject *
libgnac_converter_new(LibgnacMetadata *mdata);

void
libgnac_converter_add(LibgnacConverter  *self,
                      GFile             *file,
                      GError           **error);

void
libgnac_converter_remove(LibgnacConverter  *self,
                         GFile             *file,
                         GError           **error);

void
libgnac_converter_clear(LibgnacConverter *self);

void
libgnac_converter_start(LibgnacConverter  *self,
                        GError           **error);

void
libgnac_converter_stop(LibgnacConverter  *self,
                       GError           **error);

void
libgnac_converter_pause(LibgnacConverter  *self,
                        GError           **error);

void
libgnac_converter_resume(LibgnacConverter  *self,
                         GError           **error);

/* LibgnacConverterItem functions */

LibgnacConverterItem *
libgnac_converter_item_new(LibgnacConverter *self,
                           GFile            *source);

void
libgnac_converter_item_build(LibgnacConverterItem  *item,
                             GError               **error);

void
libgnac_converter_item_run(LibgnacConverterItem  *item,
                           GError               **error);

void
libgnac_converter_item_stop(LibgnacConverterItem  *item,
                            GError               **error);

void
libgnac_converter_item_free(LibgnacConverterItem *item);

void
libgnac_converter_item_initialize_event_source(LibgnacConverterItem *item);

void
libgnac_converter_item_clear_event_source(LibgnacConverterItem *item);

/* Callbacks */

void
libgnac_converter_eos_cb(GstBus     *bus, 
                         GstMessage *message, 
                         gpointer    user_data);
void
libgnac_converter_error_cb(GstBus     *bus, 
                           GstMessage *message, 
                           gpointer    user_data);

void
libgnac_converter_missing_plugin_result_cb(GstInstallPluginsReturn result,
                                           gpointer                user_data);

gboolean
libgnac_converter_percentage_cb(LibgnacConverterItem *item);

G_END_DECLS

#endif /* __LIBGNAC_CONVERTER_H__ */
