/*
 * $Id: gnac-playlist.c 297 2009-03-27 12:37:47Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <gio/gio.h>
#include <glib.h>
#include <glib/gi18n.h>

#include "gnac-main.h"
#include "gnac-playlist.h"

// TODO convert content to utf8!
// work with utf8, needed for all gnome programs

static gchar *
gnac_playlist_read_file(GFile *file) {
  gsize  size;
  gchar *contents;

  if (!g_file_load_contents(file, NULL, &contents, &size, NULL, NULL)) {
    return NULL;
  }

  if (size == 0) {
    g_free(contents);
    return NULL;
  }
  
  return contents;
}


void
gnac_playlist_parse_m3u(GFile *file)
{
  GFile *parent;
  GFile *child;
  gchar *contents;

  parent = g_file_get_parent(file);
  contents = gnac_playlist_read_file(file);

  if (contents) {

    gchar **lines;
    lines = g_strsplit(contents, "\n", -1);

    gint i, len;
    for (i = 0; lines[i]; i++) {
      /* skip comments and empty lines */
      if (!*lines[i] || *lines[i] == '#') continue;
      g_strchomp(lines[i]);
      len = g_utf8_strlen(lines[i], -1);
      if (!g_utf8_validate(lines[i], -1, NULL)) {
        g_printerr(_("Invalid UTF-8 filename: %s\n"), lines[i]);
      } else {
        child = g_file_get_child(parent, lines[i]);
        gnac_add_file(child);
      }
    }

    g_strfreev(lines);
    g_free(contents);
  }
}


void
gnac_playlist_parse_pls(GFile *file)
{
  GFile *parent;
  GFile *child;
  gchar  *contents;

  parent = g_file_get_parent(file);
  contents = gnac_playlist_read_file(file);

  if (contents) {
  
    gchar **lines;
    lines = g_strsplit(contents, "\n", -1);

    gint i, len;
    for (i = 0; lines[i]; i++) {
      /* skip empty lines */
      if (!*lines[i]) continue;
      gchar  *key;
      gchar **pair;
      pair = g_strsplit(lines[i], "=", 2);
      key = g_utf8_strdown(pair[0], -1);
      if (pair[1] && g_str_has_prefix(key, "file")) {
        g_strchomp(pair[1]);
        len = g_utf8_strlen(lines[i], -1);
        if (!g_utf8_validate(pair[1], -1, NULL)) {
          g_printerr(_("Invalid UTF-8 filename: %s\n"), pair[1]);
        } else {
          child = g_file_get_child(parent, pair[1]);
          gnac_add_file(child);
        }
      }

      g_free(key);
      g_strfreev(pair);
    }

    g_strfreev(lines);
    g_free(contents);
  }
}


void
gnac_playlist_parse_xspf(GFile *file)
{
  GFile *parent;
  GFile *child;
  gchar  *contents;

  parent = g_file_get_parent(file);
  contents = gnac_playlist_read_file(file);

  if (contents) {

    gchar **lines;
    lines = g_strsplit(contents, "\n", -1);

    gint i;
    for (i = 0; lines[i]; i++) {
      /* skip empty lines */
      if (!*lines[i]) continue;
      gchar *line = g_strstrip(lines[i]);
      if (g_str_has_prefix(line, "<location>")) {
        line = g_strsplit(line, ">", 2)[1];
        line = g_strsplit(line, "<", 2)[0];
        if (!g_utf8_validate(line, -1, NULL)) {
          g_printerr(_("Invalid UTF-8 filename: %s\n"), line);
        } else {
          child = g_file_get_child(parent, line);
          gnac_add_file(child);
        }
      }
    }

    g_strfreev(lines);
    g_free(contents);
  }
}
