/*
 * $Id: gnac-prefs.c 383 2009-07-31 14:27:37Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <glib/gi18n.h>

#include "gnac-file-list.h"
#include "gnac-gconf.h"
#include "gnac-prefs.h"
#include "gnac-properties.h"
#include "gnac-ui.h"
#include "gnac-utils.h"
#include "libgnac-output.h"
#include "profiles/gnac-profiles.h"

static GtkBuilder *gnac_prefs_builder = NULL;
static GtkWidget  *gnac_prefs_window;

static const gchar *tooltip_patterns = N_(
  "Patterns available:\n"
  "  <b>\%a:</b> artist\n"
  "  <b>\%b:</b> album\n"
  "  <b>\%c:</b> comment\n"
  "  <b>\%d:</b> disc number\n"
  "  <b>\%e:</b> disc count\n"
  "  <b>\%f:</b> filename\n"
  "  <b>\%g:</b> genre\n"
  "  <b>\%t:</b> title\n"
  "  <b>\%l:</b> track count\n"
  "  <b>\%n:</b> track number\n"
  "  <b>\%y:</b> year");

const gchar *rename_pattern_default[] = {
  "\%f",
  "\%t",
  "\%a - \%t",
  "\%a - \%b - \%t",
  "\%n - \%t",
  "\%n - \%a - \%t",
  "\%n - \%a - \%b - \%t",
  "\%a - \%n - \%t",
};

const gchar *folder_hierarchy_default[] = {
  "",
  "\%a",
  "\%b",
  "\%a - \%b",
  "\%a/\%b",
  "\%a/\%b (\%y)",
};

static gchar *
gnac_prefs_get_folder_hierarchy_as_str(void);

static gchar *
gnac_prefs_get_rename_pattern_as_str(void);

static const gchar *
gnac_prefs_entry_get_text(const gchar *name)
{
  GtkWidget *widget;
  widget = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, name));
  return gtk_entry_get_text(GTK_ENTRY(widget));
}


static void 
gnac_prefs_set_same_mode(void)
{
  GtkWidget *filechooserbutton;
  GtkWidget *subfolder_entry;
  GtkWidget *subfolder_eventbox;
  GtkWidget *subfolder_label;
  GtkWidget *selected_label;

  filechooserbutton = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                        "filechooserbutton"));
  subfolder_entry = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                      "subfolder_entry"));
  subfolder_eventbox = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                  "subfolder_eventbox"));
  subfolder_label = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                  "subfolder_label"));
  selected_label = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                      "selected_label"));

  gtk_event_box_set_above_child(GTK_EVENT_BOX(subfolder_eventbox), TRUE);
  gtk_widget_set_sensitive(subfolder_entry, FALSE);
  gtk_widget_set_sensitive(filechooserbutton, FALSE);
  gtk_widget_set_sensitive(selected_label, FALSE);
  gtk_widget_set_sensitive(subfolder_label, FALSE);
}


static void
gnac_prefs_set_subfolder_mode(void)
{
  GtkWidget *filechooserbutton;
  GtkWidget *subfolder_entry;
  GtkWidget *subfolder_label;
  GtkWidget *selected_label;

  filechooserbutton = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                        "filechooserbutton"));
  subfolder_entry = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                      "subfolder_entry"));
  subfolder_label = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                  "subfolder_label"));
  selected_label = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                      "selected_label"));

  gtk_widget_set_sensitive(subfolder_entry, TRUE);
  gtk_widget_set_sensitive(filechooserbutton, FALSE);
  gtk_widget_set_sensitive(subfolder_label, TRUE);
  gtk_widget_set_sensitive(selected_label, FALSE);
}


static void
gnac_prefs_set_selected_mode(void)
{
  GtkWidget *filechooserbutton;
  GtkWidget *subfolder_entry;
  GtkWidget *subfolder_eventbox;
  GtkWidget *subfolder_label;
  GtkWidget *selected_label;

  filechooserbutton = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                        "filechooserbutton"));
  subfolder_entry = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                      "subfolder_entry"));
  subfolder_eventbox = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                       "subfolder_eventbox"));
  subfolder_label = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                  "subfolder_label"));
  selected_label = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                      "selected_label"));

  gtk_event_box_set_above_child(GTK_EVENT_BOX(subfolder_eventbox), TRUE);
  gtk_widget_set_sensitive(subfolder_entry, FALSE);
  gtk_widget_set_sensitive(filechooserbutton, TRUE);
  gtk_widget_set_sensitive(subfolder_label, FALSE);
  gtk_widget_set_sensitive(selected_label, TRUE);
}


static void
gnac_prefs_gconf_retrieve(void)
{
  gchar           *str;
  GtkComboBox     *combo;
  GtkEntry        *entry;
  GtkToggleButton *check_button;
  GtkWidget       *temp;
  gint folder_type;

  /* notification icon */
  check_button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(gnac_prefs_builder, 
                                        "check_notification_icon")); 
  gtk_toggle_button_set_active(check_button, 
                  gnac_gconf_get_boolean(GNAC_GCONF_TRAY_ICON));
  /* delete original files */
  check_button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(gnac_prefs_builder, 
                                        "erase_originals_checkbutton")); 
  gtk_toggle_button_set_active(check_button, 
                  gnac_gconf_get_boolean(GNAC_GCONF_CLEAR_SOURCE));
  /* strip special characters */
  check_button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(gnac_prefs_builder, 
                                        "strip_special_checkbutton")); 
  gtk_toggle_button_set_active(check_button, 
                  gnac_gconf_get_boolean(GNAC_GCONF_STRIP_SPECIAL));
  /* output folder */
  folder_type = gnac_gconf_get_int(GNAC_GCONF_FOLDER_TYPE);
  /* output filename combo */
  combo = GTK_COMBO_BOX(gtk_builder_get_object(gnac_prefs_builder,
                                        "output_filename_combo"));
  gtk_combo_box_set_active(combo, 
                  gnac_gconf_get_int(GNAC_GCONF_RENAME_PATTERN));
  /* output filename entry */
  str = gnac_prefs_get_rename_pattern_as_str();
  entry = GTK_ENTRY(gtk_builder_get_object(gnac_prefs_builder,
                                        "output_filename_entry"));
  gtk_entry_set_text(entry, str);
  g_free(str);
  /* folder hierarchy combo */
  combo = GTK_COMBO_BOX(gtk_builder_get_object(gnac_prefs_builder,
                                        "folder_hierarchy_combo"));
  gtk_combo_box_set_active(combo, 
                  gnac_gconf_get_int(GNAC_GCONF_FOLDER_HIERARCHY));
  /* folder hierarchy entry */
  str = gnac_prefs_get_folder_hierarchy_as_str();
  entry = GTK_ENTRY(gtk_builder_get_object(gnac_prefs_builder,
                                        "folder_hierarchy_entry"));
  gtk_entry_set_text(entry, str);
  g_free(str);
  switch (folder_type) 
  {
    /* Subfolder */
    case FOLDER_SUBDIRECTORY:
      check_button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(
                            gnac_prefs_builder, "subfolder_radiobutton"));
      temp = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                            "subfolder_entry"));
      str = gnac_gconf_get_string(GNAC_GCONF_DESTINATION_DIRECTORY); 
      gtk_entry_set_text(GTK_ENTRY(temp), str);
      g_free(str);
      gtk_toggle_button_set_active(check_button, TRUE);
      gnac_prefs_set_subfolder_mode();
    break;

    /* Selected folder */
    case FOLDER_SELECTED:
      check_button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(
                            gnac_prefs_builder, "selected_radiobutton"));
      temp = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                "filechooserbutton"));
      str = gnac_gconf_get_string(GNAC_GCONF_DESTINATION_DIRECTORY);
      gtk_file_chooser_select_uri(GTK_FILE_CHOOSER(temp), str);
      g_free(str);
      gtk_toggle_button_set_active(check_button, TRUE);
      gnac_prefs_set_selected_mode();
    break;

    /* Same as source */
    case FOLDER_CURRENT:
    default:
      check_button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(
                            gnac_prefs_builder, "same_radiobutton"));
      gtk_toggle_button_set_active(check_button, TRUE);
      gnac_prefs_set_same_mode();
    break;
  }
}


GtkWidget *
gnac_prefs_window_new(void)
{
  GError *err = NULL;
  GtkWidget *prefs_window;

  gnac_prefs_builder = gtk_builder_new();

  gtk_builder_add_from_file(gnac_prefs_builder, 
                            PKGDATADIR "/gnac-pref-window.xml", &err);
  if (err != NULL) 
  {
    g_printerr(_("Unable to read file: %s\n"), err->message);
    g_error_free(err);
  }
  g_assert(err == NULL && gnac_prefs_builder != NULL);
  gtk_builder_connect_signals(gnac_prefs_builder, NULL);

  prefs_window = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                            "gnac_preference_window"));
  /* Set the parent */
  gtk_window_set_transient_for(GTK_WINDOW(prefs_window), 
                               GTK_WINDOW(gnac_ui_get_widget("main_window")));
  return prefs_window;
}


void
gnac_prefs_window_show(void)
{
  GtkWidget *notebook;
  if (!gnac_prefs_window) {
    gnac_prefs_window = gnac_prefs_window_new();
  }

  gnac_prefs_gconf_retrieve();

  /* always start with the general tab selected */
  notebook = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, "notebook"));
  gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);

  gtk_widget_show_all(gnac_prefs_window);
  gtk_window_present(GTK_WINDOW(gnac_prefs_window));
}


static gchar *
gnac_prefs_get_rename_pattern_as_str(void)
{
  gint index;
  gint size;
  GtkComboBox *combo;
  combo = GTK_COMBO_BOX(gtk_builder_get_object(gnac_prefs_builder, 
                                        "output_filename_combo"));
  index = gtk_combo_box_get_active(combo);
  size = G_N_ELEMENTS(rename_pattern_default);
  if (index < 0 || index > size)
  {
    index = 0;
  }
  if (index == size)
  {
    return gnac_gconf_get_string(GNAC_GCONF_RENAME_PATTERN_PATTERN);
  }
  return g_strdup(rename_pattern_default[index]);
}


static gchar *
gnac_prefs_get_folder_hierarchy_as_str(void)
{
  gint index;
  gint size;
  GtkComboBox *combo;
  combo = GTK_COMBO_BOX(gtk_builder_get_object(gnac_prefs_builder, 
                                        "folder_hierarchy_combo"));
  index = gtk_combo_box_get_active(combo);
  size = G_N_ELEMENTS(folder_hierarchy_default);
  if (index < 0 || index > size)
  {
    index = 0;
  }
  if (index == size)
  {
    return gnac_gconf_get_string(GNAC_GCONF_FOLDER_HIERARCHY_PATTERN);
  }
  return g_strdup(folder_hierarchy_default[index]);
}


void
gnac_prefs_tray_icon_toggled(GtkWidget *widget,
                             gpointer   data)
{
  gnac_gconf_set_boolean(GNAC_GCONF_TRAY_ICON, 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
}


void
gnac_prefs_same_radio_toggled(GtkWidget *widget,
                              gpointer   data)
{
  gnac_prefs_set_same_mode();
  gnac_gconf_set_int(GNAC_GCONF_FOLDER_TYPE, FOLDER_CURRENT);
  gnac_gconf_set_string(GNAC_GCONF_DESTINATION_DIRECTORY, "");
}


void
gnac_prefs_erase_source_toggled(GtkWidget *widget,
                                gpointer   data)
{
  gnac_gconf_set_boolean(GNAC_GCONF_CLEAR_SOURCE, 
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
}


void
gnac_prefs_strip_special_toggled(GtkWidget *widget,
                                 gpointer   data)
{
  gnac_gconf_set_boolean(GNAC_GCONF_STRIP_SPECIAL, 
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
  gnac_prefs_update_example_label(widget, data);
}


void
gnac_prefs_subfolder_radio_toggled(GtkWidget *widget,
                                   gpointer   data)
{ 
  gnac_prefs_set_subfolder_mode();
  gnac_gconf_set_int(GNAC_GCONF_FOLDER_TYPE, FOLDER_SUBDIRECTORY);
  gnac_gconf_set_string(GNAC_GCONF_DESTINATION_DIRECTORY, 
          gnac_prefs_entry_get_text("subfolder_entry"));
}


void
gnac_prefs_subfolder_editing_done(GtkWidget *widget,
                                  gpointer   data)
{
  gnac_gconf_set_string(GNAC_GCONF_DESTINATION_DIRECTORY, 
        gtk_entry_get_text(GTK_ENTRY(widget)));
}


void
gnac_prefs_selected_radio_toggled(GtkWidget *widget,
                                  gpointer   data)
{
  GtkWidget *filechooserbutton;
  filechooserbutton = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                        "filechooserbutton"));
  gnac_prefs_set_selected_mode();
  gnac_gconf_set_int(GNAC_GCONF_FOLDER_TYPE, FOLDER_SELECTED);
  gnac_gconf_set_string(GNAC_GCONF_DESTINATION_DIRECTORY, 
      gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(filechooserbutton)));
}


void
gnac_prefs_selected_uri_changed(GtkWidget *widget,
                                gpointer   data)
{
  GtkWidget *filechooserbutton;
  filechooserbutton = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                                        "filechooserbutton"));
  gnac_gconf_set_string(GNAC_GCONF_DESTINATION_DIRECTORY, 
      gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(filechooserbutton)));
}


void
gnac_prefs_update_example_label(GtkWidget *widget,
                                gpointer   data)
{
  const gchar *pattern_filename;
  gchar       *clean_label;
  gchar       *pattern;
  gchar       *preview;
  GtkWidget   *combo;
  GtkWidget   *label;

  combo = GTK_WIDGET(gtk_builder_get_object(
                      gnac_prefs_builder, "folder_hierarchy_combo"));

  label = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                            "example_label"));

  pattern_filename = gnac_prefs_entry_get_text("output_filename_entry");
  if (pattern_filename && !g_str_equal(pattern_filename, ""))
  {
    gnac_gconf_set_string(GNAC_GCONF_RENAME_PATTERN_PATTERN, pattern_filename);
  }

  if (gtk_combo_box_get_active(GTK_COMBO_BOX(combo)) != 0)
  {
    const gchar *pattern_folder;
    pattern_folder = gnac_prefs_entry_get_text("folder_hierarchy_entry");
    if (pattern_folder && !g_str_equal(pattern_folder, ""))
    {
      gnac_gconf_set_string(GNAC_GCONF_FOLDER_HIERARCHY_PATTERN, pattern_folder);
      pattern = g_strdup_printf("%s/%s", pattern_folder, pattern_filename);
    }
    else
    {
      pattern = g_strdup(pattern_filename);
    }
  }
  else
  {
    pattern = g_strdup(pattern_filename);
  }

  preview = libgnac_output_get_preview_from_pattern(pattern, 
                  gnac_gconf_get_boolean(GNAC_GCONF_STRIP_SPECIAL));
  clean_label = g_markup_printf_escaped("<small><i>%s.%s</i></small>", 
                  preview, gnac_profiles_get_extension());

  gtk_label_set_markup(GTK_LABEL(label), clean_label);

  g_free(pattern);
  g_free(preview);
  g_free(clean_label);
}


gboolean
gnac_prefs_button_press_event_cb(GtkWidget      *widget, 
                                 GdkEventButton *event, 
                                 gpointer        user_data)
{
  GtkWidget *entry;
  GType      widget_type;

  /* handle left clicks only */
  if (event->button != 1) return FALSE;

  entry = gtk_bin_get_child(GTK_BIN(user_data));
  widget_type = GTK_WIDGET_TYPE(widget);

  if (widget_type == GTK_TYPE_COMBO_BOX)
  {
    gint size;
    size = gnac_utils_get_combo_size(GTK_COMBO_BOX(widget));
    gtk_combo_box_set_active(GTK_COMBO_BOX(widget), size-1);
  }
  else if (widget_type == GTK_TYPE_RADIO_BUTTON)
  {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
  }

  gtk_event_box_set_above_child(GTK_EVENT_BOX(user_data), FALSE);
  gtk_widget_set_sensitive(entry, TRUE);
  gtk_widget_grab_focus(entry);
  
  return TRUE;
}


void
gnac_prefs_pattern_changed_cb(GtkWidget *widget,
                              gpointer   data)
{
  gint size;
  gint active;
  gchar *pattern;
  GtkWidget *filename_entry;
  GtkWidget *filename_eventbox;

  active = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
  gnac_prefs_update_example_label(widget, data);

  filename_entry = GTK_WIDGET(gtk_builder_get_object(
                      gnac_prefs_builder, "output_filename_entry"));
  filename_eventbox = GTK_WIDGET(gtk_builder_get_object(
                      gnac_prefs_builder, "output_filename_eventbox"));

  size = gnac_utils_get_combo_size(GTK_COMBO_BOX(widget));
  gtk_event_box_set_above_child(GTK_EVENT_BOX(filename_eventbox), 
                  (active != size-1));
  gtk_widget_set_sensitive(filename_entry, (active == size-1));

  pattern = gnac_prefs_get_rename_pattern_as_str();

  gtk_entry_set_text(GTK_ENTRY(filename_entry), pattern);

  gnac_gconf_set_int(GNAC_GCONF_RENAME_PATTERN, active);
  gnac_gconf_set_string(GNAC_GCONF_RENAME_PATTERN_PATTERN, pattern);

  g_free(pattern);
}


void
gnac_prefs_folder_hierarchy_changed_cb(GtkWidget *widget,
                                       gpointer   data)
{
  gint size;
  gint active;
  gchar *pattern;
  GtkWidget *folder_entry;
  GtkWidget *folder_eventbox;

  active = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
  gnac_prefs_update_example_label(widget, data);

  folder_entry = GTK_WIDGET(gtk_builder_get_object(
                    gnac_prefs_builder, "folder_hierarchy_entry"));
  folder_eventbox = GTK_WIDGET(gtk_builder_get_object(
                       gnac_prefs_builder, "folder_hierarchy_eventbox"));

  size = gnac_utils_get_combo_size(GTK_COMBO_BOX(widget));
  gtk_event_box_set_above_child(GTK_EVENT_BOX(folder_eventbox), 
                  (active != size-1));
  gtk_widget_set_sensitive(folder_entry, (active == size-1));

  pattern = gnac_prefs_get_folder_hierarchy_as_str();

  gtk_entry_set_text(GTK_ENTRY(folder_entry), pattern);

  gnac_gconf_set_int(GNAC_GCONF_FOLDER_HIERARCHY, active);
  gnac_gconf_set_string(GNAC_GCONF_FOLDER_HIERARCHY_PATTERN, pattern);

  g_free(pattern);
}


gboolean
gnac_prefs_query_tooltip_cb(GtkWidget  *entry,
                            gint        x,
                            gint        y,
                            gboolean    keyboard_mode,
                            GtkTooltip *tooltip,
                            gpointer    user_data)
{
  //if (GTK_WIDGET_IS_SENSITIVE(entry))
  //{
    gtk_tooltip_set_markup(tooltip, tooltip_patterns);
    return TRUE;
  //}
  //return FALSE;
}


void
gnac_prefs_window_hide(void)
{
  if (gnac_prefs_window) {
    gtk_widget_hide_all(gnac_prefs_window);
  }
}


void
gnac_on_close_prefs(GtkWidget *widget,
                    gpointer   data)
{
  gnac_prefs_window_hide();
}


void
gnac_prefs_destroy(void)
{  
  GtkWidget *window = NULL;

  if (gnac_prefs_builder) {
    window = GTK_WIDGET(gtk_builder_get_object(gnac_prefs_builder, 
                                       "gnac_preference_window"));
    g_object_unref(gnac_prefs_builder);
    gnac_prefs_builder = NULL;
  }

  if (window)
    gtk_widget_destroy(window);
}
