/*
 * $Id: gnac-profiles-aac.c 282 2009-03-24 23:34:35Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gnac-profiles-aac.h"

BasicFormatInfo aac_bfi =
{
  PKGDATADIR "/profiles/gnac-profiles-aac.xml",
  NULL,
  PKGDATADIR "/profiles/aac.xml",
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL
};

static UpdateTextBufferFunc update_text_buffer_call_back;

FormatModuleFuncs
gnac_profiles_aac_get_funcs(void) {
  FormatModuleFuncs funcs = {
    gnac_profiles_aac_init,
    gnac_profiles_aac_get_description,
    gnac_profiles_aac_generate_pipeline,
    (FormatModuleGenerateAudioProfileFunc) gnac_profiles_aac_generate_audio_profile,
    (FormatModuleFreeAudioProfileFunc) gnac_profiles_aac_free_audio_profile,
    (FormatModuleFullfillFieldsFunc) gnac_profiles_aac_fullfill_fields,
    (FormatModuleGetWidgetFunc) gnac_profiles_aac_get_widget,
    (FormatModuleSaveInFileFunc) gnac_profiles_aac_save_profile,
    (FormatModuleLoadSpecificPropertiesFunc) gnac_profiles_aac_load_specific_properties,
    gnac_profiles_aac_clean_up,
    NULL,
    gnac_profiles_aac_get_combo_format_name,
    gnac_profiles_aac_get_plugin_name
  };
  return funcs;
}

const gchar *
gnac_profiles_aac_init(UpdateTextBufferFunc call_back)
{
  update_text_buffer_call_back = call_back;
  GtkWidget *widget;
  GtkWidget *widget2;

  gnac_profiles_default_init(&aac_bfi, 
    G_CALLBACK(gnac_profiles_aac_generate_pipeline));

  //Bitrate
  gnac_profiles_utils_all_in_one_for_combo(&aac_bfi, 
    G_CALLBACK(gnac_profiles_aac_generate_pipeline), "combo-bitrate",
    "//variable[@id='bitrate']/*");

  //Profile
  widget = gnac_profiles_utils_all_in_one_for_combo(&aac_bfi, 
    G_CALLBACK(gnac_profiles_aac_generate_pipeline), "combo-profile",
    "//variable[@id='profile']/*");
  widget2 = GTK_WIDGET(gtk_builder_get_object(aac_bfi.builder, "label-profile")),
  gnac_profiles_utils_add_description_tooltip(&aac_bfi,
    "//variable[@id='profile']/description", widget, widget2, NULL);
  
  //Tns
  widget = gnac_profiles_utils_all_in_one_for_check(&aac_bfi,
    G_CALLBACK(gnac_profiles_aac_generate_pipeline), "checkbutton-tns",
    "//variable[@id='tns']/*");
  gnac_profiles_utils_add_description_tooltip(&aac_bfi,
    "//variable[@id='tns']/description", widget, NULL);
  
  //Outputformat
  widget = gnac_profiles_utils_all_in_one_for_combo(&aac_bfi, 
    G_CALLBACK(gnac_profiles_aac_generate_pipeline), "combo-outputformat",
    "//variable[@id='outputformat']/*");

  gnac_profiles_xml_engine_free_doc_xpath(aac_bfi.doc);
  aac_bfi.doc = NULL;

  return aac_bfi.format_id;
}

void
gnac_profiles_aac_generate_pipeline(GtkWidget *w)
{
  gchar *pipeline;

  pipeline = gnac_profiles_default_generate_pipeline(&aac_bfi);

  pipeline = gnac_profiles_utils_add_properties_combo(pipeline, &aac_bfi, 
    "combo-bitrate","combo-outputformat", "combo-profile", NULL); 

  pipeline = gnac_profiles_utils_add_properties_check(pipeline, &aac_bfi,
    "checkbutton-tns", NULL);
  
  update_text_buffer_call_back(pipeline);

  if (aac_bfi.pipeline != NULL) g_free(aac_bfi.pipeline);
  aac_bfi.pipeline = pipeline;
}

void
gnac_profiles_aac_fullfill_fields(AudioProfileAAC *profile)
{
  if (profile == NULL) {
    gnac_profiles_default_fullfill_fields(NULL, &aac_bfi);
    gnac_profiles_utils_reinit_values_combo(&aac_bfi,
      "combo-bitrate", "combo-outputformat", "combo-profile", NULL);
    gnac_profiles_utils_reinit_values_check(&aac_bfi,
      "checkbutton-tns", NULL);
  } else {
    gnac_profiles_utils_set_values_combo(&aac_bfi,
      "combo-bitrate", profile->bitrate,
      "combo-outputformat", profile->outputformat,
      "combo-profile", profile->profile,
      NULL);
    gnac_profiles_utils_set_values_check(&aac_bfi,
      "checkbutton-tns", profile->tns,
      NULL);
  }
}


gchar *
gnac_profiles_aac_get_combo_format_name()
{
  return gnac_profiles_default_get_combo_format_name(&aac_bfi);
}

void
gnac_profiles_aac_free_audio_profile(AudioProfileAAC *profile)
{
  gnac_profiles_utils_free_audio_profile_generic(profile->generic); 

  if (profile->bitrate != NULL) g_free(profile->bitrate);
  if (profile->outputformat != NULL) g_free(profile->outputformat);
  if (profile->profile != NULL) g_free(profile->profile);
  if (profile->tns != NULL) g_free(profile->tns);
  g_free(profile);
}

AudioProfileAAC *
gnac_profiles_aac_generate_audio_profile(GError **error)
{
  AudioProfileAAC *profile;
  AudioProfileGeneric *generic;

  generic = gnac_profiles_default_generate_audio_profile(&aac_bfi);
  profile = g_malloc(sizeof(AudioProfileAAC));
  profile->generic = generic;

  gnac_profiles_utils_get_values_combo_and_set(&aac_bfi,
    "combo-bitrate", &profile->bitrate,
    "combo-outputformat", &profile->outputformat,
    "combo-profile", &profile->profile,
    NULL);
 
  gnac_profiles_utils_get_values_check_and_set(&aac_bfi,
    "checkbutton-tns", &profile->tns,
    NULL);

  return profile;
}


GtkWidget *
gnac_profiles_aac_get_widget(AudioProfileAAC *profile)
{
  if (profile != NULL) {
    gnac_profiles_aac_fullfill_fields(profile);
  }
  return gnac_profiles_default_get_widget(profile,&aac_bfi);
}

void
gnac_profiles_aac_save_profile(AudioProfileAAC *profile)
{
  XMLDoc *doc;
  
  doc = gnac_profiles_default_save_profile(profile->generic, &aac_bfi);
  gnac_profiles_utils_save_specific_properties_in_file(doc, 
    "bitrate", profile->bitrate,
    "outputformat", profile->outputformat,
    "profile", profile->profile,
    "tns", profile->tns,
    NULL);

  gnac_profiles_xml_engine_save_doc(doc, profile->generic->name);
  gnac_profiles_xml_engine_free_doc_xpath(doc);
}


AudioProfileAAC*
gnac_profiles_aac_load_specific_properties(XMLDoc *doc, AudioProfileGeneric *generic)
{
  AudioProfileAAC *profile;
  
  profile = g_malloc(sizeof(AudioProfileAAC));
  profile->generic = generic;
  gnac_profiles_utils_load_saved_profile(doc, "/audio-profile/format-specific/",
    "bitrate", &profile->bitrate,
    "outputformat", &profile->outputformat,
    "profile", &profile->profile,
    "tns", &profile->tns,
    NULL);
  
  return profile;
}

void
gnac_profiles_aac_clean_up(void)
{
  gnac_profiles_default_clean_up(&aac_bfi);
  gnac_profiles_utils_free_combo_values(&aac_bfi, 
    "combo-bitrate", "combo-outputformat", "combo-profile", NULL);
  gnac_profiles_utils_free_check_values(&aac_bfi,
    "checkbutton-tns", NULL);
  gnac_profiles_utils_free_basic_format_info_content(&aac_bfi);
}

const gchar *
gnac_profiles_aac_get_plugin_name(void)
{
  return aac_bfi.format_plugin_name;
}

const gchar *
gnac_profiles_aac_get_description(void)
{
  return aac_bfi.description;
}
