/*
 * $Id: gnac-profiles-lame.c 379 2009-07-22 17:35:09Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "gnac-profiles-lame.h"

BasicFormatInfo lame_bfi =
{
  PKGDATADIR "/profiles/gnac-profiles-lame.xml",
  NULL,
  PKGDATADIR "/profiles/mp3-lame.xml",
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL
};

static void
gnac_profiles_lame_vbr_on_changed(GtkComboBox *widget);

static void
gnac_profiles_lame_advanced_bitrate_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data);

static AudioProfileLame *
gnac_profiles_lame_allocate_audio_profile(void);

static UpdateTextBufferFunc update_text_buffer_call_back;

FormatModuleFuncs
gnac_profiles_lame_get_funcs(void) {
  FormatModuleFuncs funcs = {
    gnac_profiles_lame_init,
    gnac_profiles_lame_get_description,
    gnac_profiles_lame_generate_pipeline,
    (FormatModuleGenerateAudioProfileFunc) gnac_profiles_lame_generate_audio_profile,
    (FormatModuleFreeAudioProfileFunc) gnac_profiles_lame_free_audio_profile,
    (FormatModuleFullfillFieldsFunc) gnac_profiles_lame_fullfill_fields,
    (FormatModuleGetWidgetFunc) gnac_profiles_lame_get_widget,
    (FormatModuleSaveInFileFunc) gnac_profiles_lame_save_profile,
    (FormatModuleLoadSpecificPropertiesFunc) gnac_profiles_lame_load_specific_properties,
    gnac_profiles_lame_clean_up,
    NULL,
    gnac_profiles_lame_get_combo_format_name,
    gnac_profiles_lame_get_plugin_name
  };
  return funcs;
}

const gchar *
gnac_profiles_lame_init(UpdateTextBufferFunc call_back)
{ 
  
  GtkWidget *widget;
  GtkWidget *widget2;

  gnac_profiles_default_init(&lame_bfi,
    G_CALLBACK(gnac_profiles_lame_generate_pipeline));

  update_text_buffer_call_back = call_back;
  
  //Bitrate
  gnac_profiles_utils_all_in_one_for_combo(&lame_bfi, 
    G_CALLBACK(gnac_profiles_lame_generate_pipeline), "combo-bitrate",
    "//variable[@id='bitrate']/*");
  
  //Quality
  gnac_profiles_utils_all_in_one_for_slider(&lame_bfi,
    G_CALLBACK(gnac_profiles_lame_generate_pipeline), "slider-vbr-quality",
    "//variable[@id='quality']/*");

  //compression ratio
  widget = gnac_profiles_utils_all_in_one_for_slider(&lame_bfi,
    G_CALLBACK(gnac_profiles_lame_generate_pipeline), "slider-compression-ratio",
    "//variable[@id='compression-ratio']/*");
  widget2 = GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "label-compression-ratio")),
  gnac_profiles_utils_add_description_tooltip(&lame_bfi,
    "//variable[@id='compression-ratio']/description", widget, widget2, NULL);

  //Preset
  gnac_profiles_utils_all_in_one_for_combo(&lame_bfi,
    G_CALLBACK(gnac_profiles_lame_generate_pipeline), "combo-preset",
    "//variable[@id='preset']/*");

  //Abr bitrate
  widget = gnac_profiles_utils_all_in_one_for_slider(&lame_bfi,
    G_CALLBACK(gnac_profiles_lame_generate_pipeline), "slider-mean-abr",
    "//variable[@id='mean-bitrate']/*");
  
  //Mode
  gnac_profiles_utils_all_in_one_for_combo(&lame_bfi,
    G_CALLBACK(gnac_profiles_lame_generate_pipeline), "combo-mode",
    "//variable[@id='mode']/*");
  
  //Mean bitrate
  widget = gnac_profiles_utils_all_in_one_for_slider(&lame_bfi,
    G_CALLBACK(gnac_profiles_lame_generate_pipeline), "slider-mean-vbr",
    "//variable[@id='mean-bitrate']/*");
  gnac_profiles_utils_add_toggle_signal(&lame_bfi, "checkbutton-mean-bitrate",
    G_CALLBACK(gnac_profiles_lame_advanced_bitrate_on_toggle), widget);
  
  //Min bitrate
  widget = gnac_profiles_utils_all_in_one_for_combo(&lame_bfi,
    G_CALLBACK(gnac_profiles_lame_generate_pipeline), "combo-min-vbr",
    "//variable[@id='min-bitrate']/*");
  gnac_profiles_utils_add_toggle_signal(&lame_bfi, "checkbutton-min-bitrate",
    G_CALLBACK(gnac_profiles_lame_advanced_bitrate_on_toggle), widget);

  //Max bitrate
  widget = gnac_profiles_utils_all_in_one_for_combo(&lame_bfi,
    G_CALLBACK(gnac_profiles_lame_generate_pipeline), "combo-max-vbr",
    "//variable[@id='max-bitrate']/*");
  gnac_profiles_utils_add_toggle_signal(&lame_bfi, "checkbutton-max-bitrate",
    G_CALLBACK(gnac_profiles_lame_advanced_bitrate_on_toggle), widget);

  //Encoding mode
  widget = gnac_profiles_utils_all_in_one_for_combo(&lame_bfi,
    G_CALLBACK(gnac_profiles_lame_generate_pipeline), "combo-encoding-mode",
    "//variable[@id='encoding-mode']/*");
 
  gnac_profiles_lame_vbr_on_changed(GTK_COMBO_BOX(widget));

  gnac_profiles_xml_engine_free_doc_xpath(lame_bfi.doc);
  lame_bfi.doc = NULL;

  return lame_bfi.format_id;
}

static void
gnac_profiles_lame_vbr_on_changed(GtkComboBox *widget)
{
  gint index;
  gint i;

  GtkWidget *widgets_tab[16] = {
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "label-vbr")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "hbox-vbr")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "label-cbr")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "hbox-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "label-preset")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "hbox-preset")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "label-compression-ratio")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "slider-compression-ratio")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "label-mean-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "slider-mean-abr")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "checkbutton-mean-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "slider-mean-vbr")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "checkbutton-min-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "combo-min-vbr")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "checkbutton-max-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "combo-max-vbr"))
  };
  
  index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

  for (i = 0; i<16; ++i) {
    if  (   ((index == 2) && ((i >= 10) || (i <= 1)))
        ||  ((index == 0) && ((i == 2) || (i == 3)))
        ||  ((index == 1) && (((i > 11) || (i == 8) || (i == 9))))
        ||  ((index == 3) && ((i == 4) || (i == 5)))
        ||  ((index == 4) && ((i == 6) || (i == 7)))) {
      gtk_widget_show(widgets_tab[i]);
    } else {
      gtk_widget_hide(widgets_tab[i]);
    }
  }
}

void
gnac_profiles_lame_generate_pipeline(GtkWidget *w)
{
  GtkWidget *widget; 
  gchar *pipeline;
  gint index;

  pipeline = gnac_profiles_default_generate_pipeline(&lame_bfi);

  widget = GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "combo-encoding-mode"));
  
  if (w == widget) {
    gnac_profiles_lame_vbr_on_changed(GTK_COMBO_BOX(w));
  }

  index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

  //Mode
  pipeline = gnac_profiles_utils_add_properties_combo(pipeline, &lame_bfi, 
    "combo-mode", NULL);
  
  switch (index) {
    case 0:
      //CBR
      pipeline = gnac_profiles_utils_add_properties_combo(pipeline, &lame_bfi, 
        "combo-encoding-mode", "combo-bitrate", NULL);
      break;
    case 1:
      //abr
      pipeline = gnac_profiles_utils_add_properties_combo(pipeline, &lame_bfi, 
        "combo-encoding-mode", NULL);
      pipeline =  gnac_profiles_utils_add_properties_slider(pipeline, &lame_bfi,
        "slider-mean-abr", NULL);
      
      //min bitrate max bitrate
      pipeline =  gnac_profiles_utils_add_properties_checked_combo(pipeline, &lame_bfi,
        "combo-min-vbr", "checkbutton-min-bitrate",
        "combo-max-vbr", "checkbutton-max-bitrate",
         NULL );
      break;
    case 2:
      //VBR
      pipeline = gnac_profiles_utils_add_properties_combo(pipeline, &lame_bfi, 
        "combo-encoding-mode", NULL);
      pipeline = gnac_profiles_utils_add_properties_slider(pipeline, &lame_bfi, 
       "slider-vbr-quality", NULL); 

      //mean bitrate
      pipeline =  gnac_profiles_utils_add_properties_checked_slider(pipeline, &lame_bfi,
        "slider-mean-vbr", "checkbutton-mean-bitrate", NULL);

      //min bitrate max bitrate
      pipeline =  gnac_profiles_utils_add_properties_checked_combo(pipeline, &lame_bfi,
        "combo-min-vbr", "checkbutton-min-bitrate",
        "combo-max-vbr", "checkbutton-max-bitrate",
         NULL );
      break;
    case 3:
      //Presets
      pipeline = gnac_profiles_utils_add_properties_combo(pipeline, &lame_bfi, 
        "combo-preset", NULL);
      break;
    case 4:
      //compression ratio
      pipeline = gnac_profiles_utils_add_properties_slider(pipeline, &lame_bfi, 
        "slider-compression-ratio", NULL);
      break;
    default:
      break;
  }


  pipeline = gnac_profiles_utils_add_pipe(pipeline,lame_bfi.pipeline_multiplexers);
  update_text_buffer_call_back(pipeline);

  if (lame_bfi.pipeline != NULL) g_free(lame_bfi.pipeline);
  lame_bfi.pipeline = pipeline;
}

static void
gnac_profiles_lame_advanced_bitrate_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data)
{
  gnac_profiles_utils_on_toggle_optionnal_property(togglebutton,user_data);
  gnac_profiles_lame_generate_pipeline(GTK_WIDGET(togglebutton));
}

void
gnac_profiles_lame_fullfill_fields(AudioProfileLame *profile)
{
  if (profile == NULL) {
    gnac_profiles_utils_reinit_values_combo(&lame_bfi,
      "combo-mode","combo-bitrate", "combo-min-vbr", "combo-max-vbr",
      "combo-preset", "combo-encoding-mode", NULL);
    gnac_profiles_utils_reinit_values_slider(&lame_bfi,
      "slider-vbr-quality", "slider-compression-ratio", "slider-mean-vbr",
      "slider-mean-abr", NULL);
    gnac_profiles_utils_set_active_toggle_button(&lame_bfi, FALSE,
      "checkbutton-mean-bitrate", "checkbutton-min-bitrate",
      "checkbutton-max-bitrate", NULL);
  } else {
    gnac_profiles_utils_set_values_combo(&lame_bfi,
      "combo-encoding-mode", profile->encoding_mode,
      "combo-mode", profile->mode,
    NULL);

    if (profile->bitrate != NULL) {
      gnac_profiles_utils_set_values_combo(&lame_bfi,
        "combo-bitrate", profile->bitrate, NULL); 
    } else if (profile->preset != NULL) {
      gnac_profiles_utils_set_values_combo(&lame_bfi,
        "combo-preset", profile->preset, NULL); 
    } else if (profile->mean_abr != -1) {

      gnac_profiles_utils_set_values_slider(&lame_bfi,"slider-mean-abr",
        profile->mean_abr, NULL);
      
      gnac_profiles_utils_set_values_checked_combo(&lame_bfi,
        "combo-min-vbr", "checkbutton-min-bitrate", profile->min_bitrate,
        "combo-max-vbr", "checkbutton-max-bitrate", profile->max_bitrate,
        NULL );

    } else if (profile->quality != -1) {

      gnac_profiles_utils_set_values_slider(&lame_bfi,"slider-vbr-quality",
        profile->quality, NULL);

      gnac_profiles_utils_set_values_checked_combo(&lame_bfi,
        "combo-min-vbr", "checkbutton-min-bitrate", profile->min_bitrate,
        "combo-max-vbr", "checkbutton-max-bitrate", profile->max_bitrate,
        NULL );
      gnac_profiles_utils_set_values_checked_slider(&lame_bfi,
        "slider-mean-vbr", "checkbutton-mean-bitrate", profile->mean_bitrate,
        NULL);
    } else if (profile->compression_ratio != -1) {
      gnac_profiles_utils_set_values_slider(&lame_bfi,"slider-compression-ratio",
        profile->compression_ratio, NULL);
    }
  }
}

gchar *
gnac_profiles_lame_get_combo_format_name()
{
  return gnac_profiles_default_get_combo_format_name(&lame_bfi);
}

AudioProfileLame *
gnac_profiles_lame_allocate_audio_profile(void)
{
  AudioProfileLame *profile;

  profile = g_malloc(sizeof(AudioProfileLame));
  
  profile->encoding_mode = NULL;
  profile->mode = NULL;
  profile->bitrate = NULL;
  profile->preset = NULL;
  profile->quality = -1;
  profile->compression_ratio = -1;
  profile->mean_bitrate = -1;
  profile->mean_abr = -1;
  profile->min_bitrate = NULL;
  profile->max_bitrate = NULL;

  return profile;
}


void
gnac_profiles_lame_free_audio_profile(AudioProfileLame *profile)
{
  gnac_profiles_utils_free_audio_profile_generic(profile->generic); 
  
  if (profile->encoding_mode != NULL) g_free(profile->encoding_mode);
  if (profile->mode != NULL) g_free(profile->mode);
  if (profile->bitrate != NULL) g_free(profile->bitrate);
  if (profile->min_bitrate != NULL) g_free(profile->min_bitrate);
  if (profile->max_bitrate != NULL) g_free(profile->max_bitrate);
  if (profile->preset != NULL) g_free(profile->preset);
  g_free(profile);
}


AudioProfileLame *
gnac_profiles_lame_generate_audio_profile(GError **error)
{
  AudioProfileGeneric *generic;
  AudioProfileLame *profile;
  GtkWidget *widget;
  gint index;

  generic = gnac_profiles_default_generate_audio_profile(&lame_bfi);

  profile = gnac_profiles_lame_allocate_audio_profile();
  profile->generic = generic;
  
  gnac_profiles_utils_get_values_combo_and_set(&lame_bfi,
    "combo-encoding-mode", &profile->encoding_mode,
    "combo-mode", &profile->mode, NULL);

  widget = GTK_WIDGET(gtk_builder_get_object(lame_bfi.builder, "combo-encoding-mode"));
  index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
  switch (index) {
    case 0:
      gnac_profiles_utils_get_values_combo_and_set(&lame_bfi,
        "combo-bitrate", &profile->bitrate, NULL);
      break;
    case 1:
      gnac_profiles_utils_get_values_slider_and_set(&lame_bfi,
        "slider-mean-abr", &profile->mean_abr, NULL);
      
      //min bitrate max bitrate
      gnac_profiles_utils_get_values_checked_combo_and_set(&lame_bfi,
        "combo-min-vbr", "checkbutton-min-bitrate", &profile->min_bitrate,
        "combo-max-vbr", "checkbutton-max-bitrate", &profile->max_bitrate,
         NULL );
      break;
    case 2:
      //VBR
      gnac_profiles_utils_get_values_slider_and_set(&lame_bfi,
        "slider-vbr-quality", &profile->quality, NULL);

      //mean bitrate
      gnac_profiles_utils_get_values_checked_slider_and_set(&lame_bfi,
        "slider-mean-vbr", "checkbutton-mean-bitrate", &profile->mean_bitrate,
        NULL);

      //min bitrate max bitrate
      gnac_profiles_utils_get_values_checked_combo_and_set(&lame_bfi,
        "combo-min-vbr", "checkbutton-min-bitrate", &profile->min_bitrate,
        "combo-max-vbr", "checkbutton-max-bitrate", &profile->max_bitrate,
         NULL );
      break;
    case 3:
      //Presets
      gnac_profiles_utils_get_values_combo_and_set(&lame_bfi,
        "combo-preset", &profile->preset, NULL);
      break;
    case 4:
      //Compression ratio
      gnac_profiles_utils_get_values_slider_and_set(&lame_bfi,
        "slider-compression-ratio", &profile->compression_ratio, NULL);
      break;
    default:
      break;
  }
  return profile;
}

GtkWidget *
gnac_profiles_lame_get_widget(AudioProfileLame *profile)
{
  if (profile != NULL) {
    gnac_profiles_lame_fullfill_fields(profile);
  }
  return gnac_profiles_default_get_widget(profile,&lame_bfi);
}

void
gnac_profiles_lame_save_profile(AudioProfileLame *profile)
{
  XMLDoc *doc;
  gchar *quality_str = NULL;
  gchar *mean_bitrate_str = NULL;
  gchar *mean_abr_str = NULL;
  gchar *compression_ratio_str = NULL;
  
  if (profile->quality != -1) {
    quality_str = gnac_profiles_utils_gdouble_to_gchararray(
      profile->quality);

    if (profile->mean_bitrate != -1) {
      mean_bitrate_str = gnac_profiles_utils_gdouble_to_gchararray(
        profile->mean_bitrate);
    }
  } else if (profile->mean_abr != -1) {
    mean_abr_str = gnac_profiles_utils_gdouble_to_gchararray(
      profile->mean_abr);
  } else if (profile->compression_ratio != -1) {
    compression_ratio_str = gnac_profiles_utils_gdouble_to_gchararray(
      profile->compression_ratio);
  }
  

  doc = gnac_profiles_default_save_profile(profile->generic, &lame_bfi);
  gnac_profiles_utils_save_specific_properties_in_file(doc, 
    "encoding-mode", profile->encoding_mode,
    "bitrate", (profile->bitrate == NULL) ? "" : profile->bitrate,
    "quality", (quality_str == NULL) ? "" : quality_str,
    "compression-ratio", (compression_ratio_str == NULL) ? "" : compression_ratio_str,
    "preset", (profile->preset == NULL) ? "" : profile->preset,
    "mean-abr", (mean_abr_str == NULL) ? "" : mean_abr_str,
    "mean-bitrate", (mean_bitrate_str == NULL) ? "" : mean_bitrate_str,
    "min-bitrate", (profile->min_bitrate == NULL) ? "" : profile->min_bitrate,
    "max-bitrate", (profile->max_bitrate == NULL) ? "" : profile->max_bitrate,
    "mode", profile->mode,
    NULL
    ); 
  gnac_profiles_xml_engine_save_doc(doc, profile->generic->name);
  gnac_profiles_xml_engine_free_doc_xpath(doc);
  if (quality_str != NULL) g_free(quality_str);
  if (mean_bitrate_str != NULL) g_free(mean_bitrate_str);
  if (compression_ratio_str != NULL) g_free(compression_ratio_str);
}


AudioProfileLame *
gnac_profiles_lame_load_specific_properties(XMLDoc *doc, AudioProfileGeneric *generic)
{
  AudioProfileLame *profile;
  gchar *quality_str = NULL;
  gchar *mean_bitrate_str = NULL;
  gchar *mean_abr_str = NULL;
  gchar *compression_ratio_str = NULL;
  
  profile = gnac_profiles_lame_allocate_audio_profile();
  profile->generic = generic;
  gnac_profiles_utils_load_saved_profile(doc, "/audio-profile/format-specific/",
    "encoding-mode", &profile->encoding_mode,
    "bitrate",  &profile->bitrate,
    "quality",  &quality_str,
    "compression-ratio",  &compression_ratio_str,
    "preset", &profile->preset,
    "mean-abr",  &mean_abr_str,
    "mean-bitrate",  &mean_bitrate_str,
    "min-bitrate",  &profile->min_bitrate,
    "max-bitrate",  &profile->max_bitrate,
    "mode", &profile->mode,
    NULL
  );

  if (quality_str != NULL) {
    profile->quality = gnac_profiles_utils_gchararray_to_gdouble(quality_str);
    g_free(quality_str);
    
    if (mean_bitrate_str != NULL) {
      profile->mean_bitrate = gnac_profiles_utils_gchararray_to_gdouble(
        mean_bitrate_str);
      g_free(mean_bitrate_str);
    }
  } else if (mean_abr_str != NULL){
    profile->mean_abr = gnac_profiles_utils_gchararray_to_gdouble(mean_abr_str);
    g_free(mean_abr_str);
  } else if (compression_ratio_str != NULL) {
    profile->compression_ratio = gnac_profiles_utils_gchararray_to_gdouble(
      compression_ratio_str);
    g_free(compression_ratio_str);
  }

  return profile;
}

void
gnac_profiles_lame_clean_up(void) {
  gnac_profiles_default_clean_up(&lame_bfi);
  gnac_profiles_utils_free_combo_values(&lame_bfi,
      "combo-mode","combo-bitrate", "combo-min-vbr", "combo-max-vbr",
      "combo-preset", "combo-encoding-mode", NULL);
  gnac_profiles_utils_free_slider_values(&lame_bfi,
      "slider-vbr-quality", "slider-compression-ratio", 
      "slider-mean-vbr", "slider-mean-abr", NULL);
  gnac_profiles_utils_free_basic_format_info_content(&lame_bfi);
}

const gchar *
gnac_profiles_lame_get_plugin_name(void)
{
  return lame_bfi.format_plugin_name;
}

const gchar *
gnac_profiles_lame_get_description(void)
{
  return lame_bfi.description;
}
