/*
 * $Id: gnac-profiles-speex.c 282 2009-03-24 23:34:35Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gnac-profiles-speex.h"

BasicFormatInfo speex_bfi =
{
  PKGDATADIR "/profiles/gnac-profiles-speex.xml",
  NULL,
  PKGDATADIR "/profiles/speex.xml",
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL
};

static void
gnac_profiles_speex_bitrate_on_changed(GtkComboBox *widget);

static void
gnac_profiles_speex_advanced_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data);

static void
gnac_profiles_speex_vad_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data);

static AudioProfileSpeex *
gnac_profiles_speex_allocate_audio_profile(void);

static UpdateTextBufferFunc update_text_buffer_call_back;

FormatModuleFuncs
gnac_profiles_speex_get_funcs(void) {
  FormatModuleFuncs funcs = {
    gnac_profiles_speex_init,
    gnac_profiles_speex_get_description,
    gnac_profiles_speex_generate_pipeline,
    (FormatModuleGenerateAudioProfileFunc) gnac_profiles_speex_generate_audio_profile,
    (FormatModuleFreeAudioProfileFunc) gnac_profiles_speex_free_audio_profile,
    (FormatModuleFullfillFieldsFunc) gnac_profiles_speex_fullfill_fields,
    (FormatModuleGetWidgetFunc) gnac_profiles_speex_get_widget,
    (FormatModuleSaveInFileFunc) gnac_profiles_speex_save_profile,
    (FormatModuleLoadSpecificPropertiesFunc) gnac_profiles_speex_load_specific_properties,
    gnac_profiles_speex_clean_up,
    NULL,
    gnac_profiles_speex_get_combo_format_name,
    gnac_profiles_speex_get_plugin_name
  };
  return funcs;
}

const gchar *
gnac_profiles_speex_init(UpdateTextBufferFunc call_back)
{
  GtkWidget *widget;
  GtkWidget *widget2;

  update_text_buffer_call_back = call_back;

  gnac_profiles_default_init(&speex_bfi, 
    G_CALLBACK(gnac_profiles_speex_generate_pipeline));

  //Bitrate
  widget = gnac_profiles_utils_all_in_one_for_slider(&speex_bfi, 
    G_CALLBACK(gnac_profiles_speex_generate_pipeline), "slider-bitrate-cbr",
    "//variable[@id='bitrate']/*");
  gnac_profiles_utils_add_toggle_signal(&speex_bfi, "checkbutton-bitrate-cbr",
    G_CALLBACK(gnac_profiles_speex_advanced_on_toggle), widget);

  //Bitrate-abr
  gnac_profiles_utils_all_in_one_for_slider(&speex_bfi, 
    G_CALLBACK(gnac_profiles_speex_generate_pipeline), "slider-bitrate-abr",
    "//variable[@id='abr']/*");
  
  //Quality-cbr
  widget = gnac_profiles_utils_all_in_one_for_slider(&speex_bfi, 
    G_CALLBACK(gnac_profiles_speex_generate_pipeline), "slider-quality-cbr",
    "//variable[@id='quality-cbr']/*");

  //Quality-vbr
  widget = gnac_profiles_utils_all_in_one_for_slider(&speex_bfi, 
    G_CALLBACK(gnac_profiles_speex_generate_pipeline), "slider-quality-vbr",
    "//variable[@id='quality-vbr']/*");
  gtk_scale_set_digits(GTK_SCALE(widget), 2);

  //vbr
  widget = gnac_profiles_utils_all_in_one_for_check(&speex_bfi,
    G_CALLBACK(gnac_profiles_speex_generate_pipeline), "checkbutton-vbr",
    "//variable[@id='vbr']/*");
 
  //Complexity
  widget2 = gnac_profiles_utils_all_in_one_for_slider(&speex_bfi, 
    G_CALLBACK(gnac_profiles_speex_generate_pipeline), "slider-complexity",
    "//variable[@id='complexity']/*");
  widget = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
   "hbox-complexity"));
  gnac_profiles_utils_add_toggle_signal(&speex_bfi, "checkbutton-complexity",
    G_CALLBACK(gnac_profiles_speex_advanced_on_toggle), widget);
  widget = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
   "checkbutton-complexity"));
  gnac_profiles_utils_add_description_tooltip(&speex_bfi,
    "//variable[@id='complexity']/description", widget, widget2, NULL);

  //vad
  widget = gnac_profiles_utils_all_in_one_for_check(&speex_bfi,
    G_CALLBACK(gnac_profiles_speex_generate_pipeline), "checkbutton-vad",
    "//variable[@id='vad']/*");
  gnac_profiles_utils_add_toggle_signal(&speex_bfi, "checkbutton-vad",
    G_CALLBACK(gnac_profiles_speex_vad_on_toggle), widget);
  gnac_profiles_utils_add_description_tooltip(&speex_bfi,
    "//variable[@id='vad']/description", widget, NULL);

  //dtx
  widget = gnac_profiles_utils_all_in_one_for_check(&speex_bfi,
    G_CALLBACK(gnac_profiles_speex_generate_pipeline), "checkbutton-dtx",
    "//variable[@id='dtx']/*");
  gnac_profiles_utils_add_description_tooltip(&speex_bfi,
    "//variable[@id='dtx']/description", widget, NULL);

  //mode
  widget = gnac_profiles_utils_all_in_one_for_combo(&speex_bfi, 
    G_CALLBACK(gnac_profiles_speex_generate_pipeline), "combo-mode",
    "//variable[@id='mode']/*");
  widget2 = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
   "hbox-mode"));
  gnac_profiles_utils_add_description_tooltip(&speex_bfi,
    "//variable[@id='mode']/description", widget, widget2, NULL);
 
  //Bitrate mode
  widget = gnac_profiles_utils_all_in_one_for_combo(&speex_bfi, 
    G_CALLBACK(gnac_profiles_speex_generate_pipeline), "combo-bitrate-mode",
    "//variable[@id='bitrate-mode']/*");
  
  gnac_profiles_speex_bitrate_on_changed(GTK_COMBO_BOX(widget));
  
  gnac_profiles_xml_engine_free_doc_xpath(speex_bfi.doc);
  speex_bfi.doc = NULL;

  return speex_bfi.format_id;
}

void
gnac_profiles_speex_bitrate_on_changed(GtkComboBox *widget)
{
  gint index;
  gint i;
  
  GtkWidget *widgets_tab[8] = {
    GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder, "checkbutton-bitrate-cbr")),
    GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder, "slider-bitrate-cbr")),
    GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder, "label-quality-cbr")),
    GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder, "hbox-quality-cbr")),
    GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder, "label-bitrate-abr")),
    GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder, "slider-bitrate-abr")),
    GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder, "label-quality-vbr")),
    GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder, "hbox-quality-vbr")),
  };
  
  index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

  for (i = 0; i<8; ++i) {
    if  (   ((index == 0) && (i < 4))
        ||  ((index == 1) && ((i == 4) || (i == 5)))
        ||  ((index == 2) && ((i == 6) || (i == 7)))) {
      gtk_widget_show(widgets_tab[i]);
    } else {
      gtk_widget_hide(widgets_tab[i]);
    }
  }

  GtkWidget *vad;
  GtkWidget *dtx;

  vad = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
    "checkbutton-vad"));
  dtx = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
    "checkbutton-dtx"));
  if (index == 0) {
    gtk_widget_show(vad);
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vad))) {
      gtk_widget_show(dtx);
    } else {
      gtk_widget_hide(dtx);
    }
  } else {
    gtk_widget_hide(vad);
    gtk_widget_show(dtx);
  }

  gnac_profiles_utils_set_active_toggle_button(&speex_bfi, (index == 2),
    "checkbutton-vbr", NULL);

}


void
gnac_profiles_speex_generate_pipeline(GtkWidget *w)
{
  GtkWidget *widget; 
  gchar *pipeline;
  gint index;

  pipeline = gnac_profiles_default_generate_pipeline(&speex_bfi);
  
  widget = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder, "combo-bitrate-mode"));

  if (w == widget) {
    gnac_profiles_speex_bitrate_on_changed(GTK_COMBO_BOX(w));
  }

  index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

  switch (index) {
    case 0:
      pipeline = gnac_profiles_utils_add_properties_slider(pipeline, &speex_bfi, 
        "slider-quality-cbr", NULL); 
      widget = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
        "checkbutton-bitrate-cbr"));
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
        widget = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
          "slider-bitrate-cbr"));
        pipeline = gnac_profiles_utils_add_property_slider(pipeline, "%.0f", 1024, 
          widget); 
      }
      pipeline = gnac_profiles_utils_add_properties_check(pipeline, &speex_bfi,
        "checkbutton-vad", NULL);
      widget = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
        "checkbutton-vad"));
      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
        pipeline = gnac_profiles_utils_add_properties_check(pipeline, &speex_bfi,
          "checkbutton-dtx", NULL);
      }
      break;
    case 1:
      widget = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
        "slider-bitrate-abr"));
      pipeline = gnac_profiles_utils_add_property_slider(pipeline, "%.0f", 1024, 
        widget); 
      pipeline = gnac_profiles_utils_add_properties_check(pipeline, &speex_bfi,
        "checkbutton-dtx", NULL);
      break;
    case 2:
      pipeline = gnac_profiles_utils_add_properties_slider_format(pipeline, &speex_bfi, 
        "%.2f", "slider-quality-vbr", NULL); 
      pipeline = gnac_profiles_utils_add_properties_check(pipeline, &speex_bfi,
        "checkbutton-vbr", NULL);
      pipeline = gnac_profiles_utils_add_properties_check(pipeline, &speex_bfi,
        "checkbutton-dtx", NULL);
      break;
  }
  
  pipeline = gnac_profiles_utils_add_properties_checked_slider(pipeline, &speex_bfi,
    "slider-complexity", "checkbutton-complexity", NULL); 
 
  pipeline = gnac_profiles_utils_add_properties_combo(pipeline, &speex_bfi, 
    "combo-mode", NULL); 


  pipeline = gnac_profiles_utils_add_pipe(pipeline,speex_bfi.pipeline_multiplexers);
  update_text_buffer_call_back(pipeline);

  if (speex_bfi.pipeline != NULL) g_free(speex_bfi.pipeline);
  speex_bfi.pipeline = pipeline;

}

void
gnac_profiles_speex_advanced_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data)
{
  gnac_profiles_utils_on_toggle_optionnal_property(togglebutton,user_data);
  gnac_profiles_speex_generate_pipeline(GTK_WIDGET(togglebutton));
}

void
gnac_profiles_speex_vad_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data)
{
  GtkWidget *dtx;

  dtx = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
    "checkbutton-dtx"));
  if (gtk_toggle_button_get_active(togglebutton)) {
    gtk_widget_show(dtx);
  } else {
    gtk_widget_hide(dtx);
  }
}

void
gnac_profiles_speex_fullfill_fields(AudioProfileSpeex *profile)
{
  GtkWidget *widget;

  if (profile == NULL) {
    gnac_profiles_default_fullfill_fields(NULL, &speex_bfi);
    gnac_profiles_utils_reinit_values_combo(&speex_bfi,"combo-bitrate-mode",
      "combo-mode", NULL);
    gnac_profiles_utils_reinit_values_slider(&speex_bfi,"slider-quality-cbr",
      "slider-quality-vbr", "slider-bitrate-cbr", "slider-bitrate-abr",
      "slider-complexity", NULL);
    gnac_profiles_utils_reinit_values_check(&speex_bfi, "checkbutton-vad",
      "checkbutton-dtx", NULL);
   
    gnac_profiles_utils_set_active_toggle_button(&speex_bfi, FALSE,
      "checkbutton-complexity", "checkbutton-bitrate-cbr", NULL);
      
  } else {

    gnac_profiles_default_fullfill_fields(profile->generic,&speex_bfi);

    gnac_profiles_utils_set_values_checked_slider(&speex_bfi,
      "slider-complexity", "checkbutton-complexity", profile->complexity, NULL);
    gnac_profiles_utils_set_values_combo(&speex_bfi,
      "combo-mode", profile->mode, NULL);
    gnac_profiles_utils_set_values_check(&speex_bfi,
      "checkbutton-vad", profile->vad, "checkbutton-dtx", profile->dtx, NULL);

    widget = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder, "combo-bitrate-mode"));
    if (profile->quality_cbr != -1) {
      gnac_profiles_utils_set_values_slider(&speex_bfi, "slider-bitrate-cbr",
        profile->bitrate_cbr, NULL);
      if (profile->bitrate_cbr != 1) {
        gnac_profiles_utils_set_values_checked_slider(&speex_bfi,
          "slider-bitrate-cbr", "checkbutton-bitrate-cbr", profile->bitrate_cbr,
          NULL);
      }
      gtk_combo_box_set_active(GTK_COMBO_BOX(widget),0);
    } else if (profile->quality_vbr != -1) {
      gnac_profiles_utils_set_active_toggle_button(&speex_bfi, TRUE,
        "checkbutton-vbr", NULL);
      gnac_profiles_utils_set_values_slider(&speex_bfi, "slider-quality-vbr",
        profile->quality_vbr, NULL);
      gtk_combo_box_set_active(GTK_COMBO_BOX(widget),2);
    } else {
      gnac_profiles_utils_set_values_slider(&speex_bfi, "slider-bitrate-abr",
        profile->bitrate_abr, NULL);
      gtk_combo_box_set_active(GTK_COMBO_BOX(widget),1);
    }
  }
}


gchar *
gnac_profiles_speex_get_combo_format_name()
{
  return gnac_profiles_default_get_combo_format_name(&speex_bfi);
}

static AudioProfileSpeex *
gnac_profiles_speex_allocate_audio_profile(void)
{
  AudioProfileSpeex *profile;

  profile = g_malloc(sizeof(AudioProfileSpeex));
  
  profile->quality_cbr = -1;
  profile->quality_vbr = -1;
  profile->bitrate_cbr = -1;
  profile->bitrate_abr = -1;
  profile->mode = NULL;
  profile->complexity =  -1;
  profile->vad = NULL;
  profile->dtx = NULL;

  return profile;
}

void
gnac_profiles_speex_free_audio_profile(AudioProfileSpeex *profile)
{
  gnac_profiles_utils_free_audio_profile_generic(profile->generic); 
  
  if (profile->mode != NULL) g_free(profile->mode);
  if (profile->vad != NULL) g_free(profile->vad);
  if (profile->dtx != NULL) g_free(profile->dtx);
  g_free(profile);
}


AudioProfileSpeex *
gnac_profiles_speex_generate_audio_profile(GError **error)
{
  AudioProfileGeneric *generic;
  AudioProfileSpeex *profile;
  GtkWidget *widget;
  gint index;

  generic = gnac_profiles_default_generate_audio_profile(&speex_bfi);

  profile = gnac_profiles_speex_allocate_audio_profile();
  profile->generic = generic;

  widget = GTK_WIDGET(gtk_builder_get_object(speex_bfi.builder,
    "combo-bitrate-mode"));
  index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

  gnac_profiles_utils_get_values_combo_and_set(&speex_bfi,
    "combo-mode", &profile->mode, NULL);

  gnac_profiles_utils_get_values_checked_slider_and_set(&speex_bfi, 
    "slider-complexity", "checkbutton-complexity", &profile->complexity,
    NULL);
  gnac_profiles_utils_get_values_check_and_set(&speex_bfi,
    "checkbutton-dtx", &profile->dtx, NULL);

  switch (index) {
    case 0:
      gnac_profiles_utils_get_values_slider_and_set(&speex_bfi,
        "slider-quality-cbr", &profile->quality_cbr,
        NULL); 
      gnac_profiles_utils_get_values_checked_slider_and_set(&speex_bfi, 
        "slider-bitrate-cbr", "checkbutton-bitrate-cbr", &profile->bitrate_cbr,
        NULL);
      gnac_profiles_utils_get_values_check_and_set(&speex_bfi,
        "checkbutton-vad", &profile->vad, NULL);
      break;
    case 1:
      gnac_profiles_utils_get_values_slider_and_set(&speex_bfi,
        "slider-bitrate-abr", &profile->bitrate_abr,
        NULL);
      break;
    case 2:
      gnac_profiles_utils_get_values_slider_and_set(&speex_bfi,
        "slider-quality-vbr", &profile->quality_vbr,
        NULL);
      break;
  }
  
  return profile;
}


GtkWidget *
gnac_profiles_speex_get_widget(AudioProfileSpeex *profile)
{
  if (profile != NULL) {
    gnac_profiles_speex_fullfill_fields(profile);
  }
  return gnac_profiles_default_get_widget(profile,&speex_bfi);
}

void
gnac_profiles_speex_save_profile(AudioProfileSpeex *profile)
{
  XMLDoc *doc;
  gchar *quality_cbr_str = NULL;
  gchar *bitrate_cbr_str = NULL;
  gchar *bitrate_abr_str = NULL;
  gchar *quality_vbr_str = NULL;
  gchar *complexity_str = NULL;
  
  if (profile->quality_cbr != -1) {
    quality_cbr_str = gnac_profiles_utils_gdouble_to_gchararray(profile->quality_cbr);
    if (profile->bitrate_cbr != -1) {
      bitrate_cbr_str = gnac_profiles_utils_gdouble_to_gchararray(
        profile->bitrate_cbr);
    }
  } else if (profile->quality_vbr != -1) {
    quality_vbr_str = gnac_profiles_utils_gdouble_to_gchararray_format(profile->quality_vbr,
    "%.2f");
  } else {
    bitrate_abr_str = gnac_profiles_utils_gdouble_to_gchararray(profile->bitrate_abr);
  }
  
  if (profile->complexity != -1) {
    complexity_str = gnac_profiles_utils_gdouble_to_gchararray(profile->complexity);
  }

  doc = gnac_profiles_default_save_profile(profile->generic, &speex_bfi);
  gnac_profiles_utils_save_specific_properties_in_file(doc, 
    "bitrate-cbr", (bitrate_cbr_str == NULL) ? "" : bitrate_cbr_str,
    "quality-cbr", (quality_cbr_str == NULL) ? "" : quality_cbr_str,
    "bitrate-abr", (bitrate_abr_str == NULL) ? "" : bitrate_abr_str,
    "quality-vbr", (quality_vbr_str == NULL) ? "" : quality_vbr_str,
    "mode", profile->mode,
    "complexity", (complexity_str == NULL) ? "" : complexity_str,
    "vad", profile->vad,
    "dtx", profile->dtx,
    NULL
    ); 
  gnac_profiles_xml_engine_save_doc(doc, profile->generic->name);
  gnac_profiles_xml_engine_free_doc_xpath(doc);
  g_free(quality_cbr_str);
  g_free(quality_vbr_str);
  g_free(bitrate_cbr_str);
  g_free(bitrate_abr_str);
  g_free(complexity_str);
}


AudioProfileSpeex *
gnac_profiles_speex_load_specific_properties(XMLDoc *doc, AudioProfileGeneric *generic)
{
  AudioProfileSpeex *profile;
  gchar *quality_cbr_str;
  gchar *bitrate_cbr_str;
  gchar *bitrate_abr_str;
  gchar *quality_vbr_str;
  gchar *complexity_str;
  
  profile = gnac_profiles_speex_allocate_audio_profile();
  profile->generic = generic;
  gnac_profiles_utils_load_saved_profile(doc, "/audio-profile/format-specific/",
    "bitrate-cbr", &bitrate_cbr_str,
    "quality-cbr", &quality_cbr_str,
    "bitrate-abr", &bitrate_abr_str,
    "quality-vbr", &quality_vbr_str,
    "mode", &profile->mode,
    "complexity", &complexity_str,
    "vad", &profile->vad,
    "dtx", &profile->dtx,
    NULL
    );
  
  if (complexity_str != NULL) {
    profile->complexity = gnac_profiles_utils_gchararray_to_gdouble(complexity_str);
    g_free(complexity_str);
  }

  if (quality_cbr_str != NULL) {
    profile->quality_cbr = gnac_profiles_utils_gchararray_to_gdouble(quality_cbr_str);
    if (bitrate_cbr_str != NULL) {
      profile->bitrate_cbr = gnac_profiles_utils_gchararray_to_gdouble(bitrate_cbr_str);
      g_free(bitrate_cbr_str);
    }
    g_free(quality_cbr_str);
  } else if (quality_vbr_str != NULL) {
    profile->quality_vbr = gnac_profiles_utils_gchararray_to_gdouble(quality_vbr_str);
    g_free(quality_vbr_str);
  } else {
    profile->bitrate_abr = gnac_profiles_utils_gchararray_to_gdouble(bitrate_abr_str);
    g_free(bitrate_abr_str);
  }

  return profile;
}

void
gnac_profiles_speex_clean_up(void) {
  gnac_profiles_default_clean_up(&speex_bfi);
  gnac_profiles_utils_free_combo_values(&speex_bfi,
    "combo-mode", "combo-bitrate-mode", NULL);
  gnac_profiles_utils_free_slider_values(&speex_bfi,
    "slider-quality-cbr", "slider-quality-vbr", "slider-bitrate-cbr",
    "slider-bitrate-abr", "slider-complexity", NULL);
  gnac_profiles_utils_free_check_values(&speex_bfi,
    "checkbutton-vad", "checkbutton-dtx","checkbutton-vbr", NULL);
  gnac_profiles_utils_free_basic_format_info_content(&speex_bfi);
}

const gchar *
gnac_profiles_speex_get_plugin_name(void)
{
  return speex_bfi.format_plugin_name;
}

const gchar *
gnac_profiles_speex_get_description(void)
{
  return speex_bfi.description;
}
