/*
 * $Id: gnac-profiles-unknown.c 282 2009-03-24 23:34:35Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n.h>

#include "gnac-profiles-unknown.h"

BasicFormatInfo unknown_bfi =
{
  PKGDATADIR "/profiles/gnac-profiles-unknown.xml",
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  "custom-format",
  "audioconvert",
  N_("Custom format"),
  NULL,
  N_("<i>Use this format to define your own gstreamer pipeline</i>")
};

const gchar *base_pipeline = 
  "audioconvert ! audio/x-raw-float, rate=44100, channels=2 ! audioconvert !";

static GtkTextView *pipeline_text_view;
static GtkWidget *pipeline_box;
static gulong text_view_handler_id = 0;

static gboolean
gnac_profiles_unknown_text_view_handler(GtkWidget *widget, gpointer data);

static void
gnac_profiles_unknown_on_hide(GtkWidget *widget, gpointer data);

FormatModuleFuncs
gnac_profiles_unknown_get_funcs(void) {
  FormatModuleFuncs funcs = {
    gnac_profiles_unknown_init,
    gnac_profiles_unknown_get_description,
    gnac_profiles_unknown_generate_pipeline,
    (FormatModuleGenerateAudioProfileFunc) gnac_profiles_unknown_generate_audio_profile,
    (FormatModuleFreeAudioProfileFunc) gnac_profiles_unknown_free_audio_profile,
    (FormatModuleFullfillFieldsFunc) gnac_profiles_unknown_fullfill_fields,
    (FormatModuleGetWidgetFunc) gnac_profiles_unknown_get_widget,
    (FormatModuleSaveInFileFunc) gnac_profiles_unknown_save_profile,
    (FormatModuleLoadSpecificPropertiesFunc) gnac_profiles_unknown_load_specific_properties,
    gnac_profiles_unknown_clean_up,
    gnac_profiles_unknown_set_pipeline_text_view,
    gnac_profiles_unknown_get_combo_format_name,
    gnac_profiles_unknown_get_plugin_name
  };
  return funcs;
}

const gchar *
gnac_profiles_unknown_init(UpdateTextBufferFunc call_back)
{
  GtkWidget *widget;
  GError *error = NULL;

  unknown_bfi.builder = gtk_builder_new();
  if (!gtk_builder_add_from_file(unknown_bfi.builder,
    unknown_bfi.gtkbuilder_xml_file, &error)) {
    g_printerr("%s\n",error->message);
    g_error_free(error);
    exit(EXIT_FAILURE);
  }

  widget = GTK_WIDGET(gtk_builder_get_object(unknown_bfi.builder, "properties-alignment"));
  //g_object_ref (widget);
  //gtk_widget_unparent(widget);

  g_signal_connect (G_OBJECT (widget), "hide",
      G_CALLBACK(gnac_profiles_unknown_on_hide), NULL);

  

  return unknown_bfi.format_id;
}


void
gnac_profiles_unknown_set_pipeline_text_view(GtkTextView *text_view, GtkWidget *box)
{
  pipeline_text_view = text_view;
  pipeline_box = box;
}

void
gnac_profiles_unknown_generate_pipeline(GtkWidget *w)
{}

void
gnac_profiles_unknown_fullfill_fields(AudioProfileGeneric *profile)
{
  GtkWidget *widget;
  GtkTextBuffer *text_buffer;

  widget = GTK_WIDGET(gtk_builder_get_object(unknown_bfi.builder,
    "entry-extension"));
  if (profile == NULL) {
    gtk_entry_set_text(GTK_ENTRY(widget), "");
    if (unknown_bfi.pipeline != NULL) {
      g_free(unknown_bfi.pipeline);
      unknown_bfi.pipeline = NULL;
    }
  } else {
    gtk_entry_set_text(GTK_ENTRY(widget), profile->extension);

    text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pipeline_text_view));
    gtk_text_buffer_set_text(text_buffer, profile->pipeline,
      strlen(profile->pipeline));
    unknown_bfi.pipeline = g_strdup(profile->pipeline);
  }

}

gchar *
gnac_profiles_unknown_get_combo_format_name()
{
  return g_strdup(_(unknown_bfi.format_name));
}

void
gnac_profiles_unknown_free_audio_profile(AudioProfileGeneric *profile)
{
  gnac_profiles_utils_free_audio_profile_generic(profile); 
}


AudioProfileGeneric *
gnac_profiles_unknown_generate_audio_profile(GError **error)
{
  AudioProfileGeneric *profile;
  GtkWidget *widget;
  const gchar *extension;

  widget = GTK_WIDGET(gtk_builder_get_object(unknown_bfi.builder,
    "entry-extension"));
  extension = gtk_entry_get_text(GTK_ENTRY(widget));
  if (!g_strcmp0(extension, "")) {
    if ((error) && (*error == NULL)) { 
      *error = g_error_new(g_quark_from_static_string("gnac")
        ,0, _("The extension field must be non-empty")); 
    }
    return NULL;
  } else {
    profile = gnac_profiles_utils_allocate_audio_profile_generic();
    
    profile->extension = g_strdup(extension);
    profile->format_id = g_strdup(unknown_bfi.format_id);
    profile->format_name = g_strdup(unknown_bfi.format_name);
    profile->channels = g_strdup("-1");
    profile->rate = g_strdup("-1");
  
    return profile;
  }
}


GtkWidget *
gnac_profiles_unknown_get_widget(AudioProfileGeneric *profile)
{
  gtk_text_view_set_editable(pipeline_text_view, TRUE);

  if (profile != NULL) {
    gnac_profiles_unknown_fullfill_fields(profile);
  }

  if (!g_signal_handler_is_connected(G_OBJECT(pipeline_text_view), text_view_handler_id)) {
    GtkTextBuffer *text_buffer;  
   
    #ifndef GNOME_ENABLE_DEBUG 
    gtk_widget_show(GTK_WIDGET(pipeline_box));
    #endif /* GNOME_ENABLE_DEBUG */

    text_view_handler_id = g_signal_connect(G_OBJECT(pipeline_text_view), "focus-out-event",
      G_CALLBACK(gnac_profiles_unknown_text_view_handler), NULL);
  
    text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pipeline_text_view));
    
    if (unknown_bfi.pipeline  != NULL) {
      gtk_text_buffer_set_text(text_buffer, unknown_bfi.pipeline,
        strlen(unknown_bfi.pipeline));
    } else {
      gtk_text_buffer_set_text(text_buffer, base_pipeline,
        strlen(base_pipeline));
    }
  }

  return gnac_profiles_default_get_widget(profile,&unknown_bfi);
}

void
gnac_profiles_unknown_save_profile(AudioProfileGeneric *profile)
{
  XMLDoc *doc;

  doc = gnac_profiles_default_save_profile((AudioProfileGeneric*) profile, &unknown_bfi);
  gnac_profiles_xml_engine_save_doc(doc, profile->generic->name);
  gnac_profiles_xml_engine_free_doc_xpath(doc);
}


AudioProfileGeneric *
gnac_profiles_unknown_load_specific_properties(XMLDoc *doc, AudioProfileGeneric *generic)
{
  DISCARD_PARAMETER(doc);

  return generic;
}

void
gnac_profiles_unknown_clean_up(void) {
  GtkWidget *widget;

  widget = GTK_WIDGET(gtk_builder_get_object(unknown_bfi.builder, "properties-alignment"));
  //gtk_widget_unref(widget);
  if (unknown_bfi.pipeline != NULL) g_free(unknown_bfi.pipeline);
  g_object_unref(unknown_bfi.builder);
  gtk_widget_destroy(widget);
}

const gchar *
gnac_profiles_unknown_get_plugin_name(void)
{
  return unknown_bfi.format_plugin_name;
}

const gchar *
gnac_profiles_unknown_get_description(void)
{
  return _(unknown_bfi.description);
}

static gboolean
gnac_profiles_unknown_text_view_handler(GtkWidget *widget, gpointer data)
{
  GtkTextBuffer *text_buffer;
  GtkTextIter start;
  GtkTextIter end;
  
  if (unknown_bfi.pipeline != NULL) g_free(unknown_bfi.pipeline);

  text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(pipeline_text_view));
  gtk_text_buffer_get_start_iter(text_buffer,&start);
  gtk_text_buffer_get_end_iter(text_buffer,&end);
  unknown_bfi.pipeline = gtk_text_buffer_get_text(text_buffer,&start,&end,FALSE);

  return FALSE;
}

static void
gnac_profiles_unknown_on_hide(GtkWidget *widget, gpointer data)
{
  if (g_signal_handler_is_connected(G_OBJECT(pipeline_text_view),  text_view_handler_id)) {
    g_signal_handler_disconnect(G_OBJECT(pipeline_text_view), text_view_handler_id);
  }

}
