/*
 * $Id: gnac-profiles-wavpack.c 282 2009-03-24 23:34:35Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "gnac-profiles-wavpack.h"

BasicFormatInfo wavpack_bfi =
{
  PKGDATADIR "/profiles/gnac-profiles-wavpack.xml",
  NULL,
  PKGDATADIR "/profiles/wavpack.xml",
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL
};

static void
gnac_profiles_wavpack_bitrate_control_on_changed(GtkComboBox *widget);

static void
gnac_profiles_wavpack_bitrate_control_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data);

static void
gnac_profiles_wavpack_joint_stereo_mode_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data);

static AudioProfileWavpack *
gnac_profiles_wavpack_allocate_audio_profile(void);

static UpdateTextBufferFunc update_text_buffer_call_back;

FormatModuleFuncs
gnac_profiles_wavpack_get_funcs(void) {
  FormatModuleFuncs funcs = {
    gnac_profiles_wavpack_init,
    gnac_profiles_wavpack_get_description,
    gnac_profiles_wavpack_generate_pipeline,
    (FormatModuleGenerateAudioProfileFunc) gnac_profiles_wavpack_generate_audio_profile,
    (FormatModuleFreeAudioProfileFunc) gnac_profiles_wavpack_free_audio_profile,
    (FormatModuleFullfillFieldsFunc) gnac_profiles_wavpack_fullfill_fields,
    (FormatModuleGetWidgetFunc) gnac_profiles_wavpack_get_widget,
    (FormatModuleSaveInFileFunc) gnac_profiles_wavpack_save_profile,
    (FormatModuleLoadSpecificPropertiesFunc) gnac_profiles_wavpack_load_specific_properties,
    gnac_profiles_wavpack_clean_up,
    NULL,
    gnac_profiles_wavpack_get_combo_format_name,
    gnac_profiles_wavpack_get_plugin_name
  };
  return funcs;
}


const gchar *
gnac_profiles_wavpack_init(UpdateTextBufferFunc call_back)
{
  GtkWidget *widget;
  GtkWidget *widget2;

  update_text_buffer_call_back = call_back;

  gnac_profiles_default_init(&wavpack_bfi, 
    G_CALLBACK(gnac_profiles_wavpack_generate_pipeline));

  //Encoding mode
  gnac_profiles_utils_all_in_one_for_combo(&wavpack_bfi, 
    G_CALLBACK(gnac_profiles_wavpack_generate_pipeline), "combo-mode",
    "//variable[@id='mode']/*");

  //Bitrate
  gnac_profiles_utils_all_in_one_for_slider(&wavpack_bfi, 
    G_CALLBACK(gnac_profiles_wavpack_generate_pipeline), "slider-bitrate",
    "//variable[@id='bitrate']/*");

  //Bits per sample
  gnac_profiles_utils_all_in_one_for_slider(&wavpack_bfi, 
    G_CALLBACK(gnac_profiles_wavpack_generate_pipeline), "slider-bits-per-sample",
    "//variable[@id='bits-per-sample']/*");
 
  //Extra processing
  widget = gnac_profiles_utils_all_in_one_for_slider(&wavpack_bfi, 
    G_CALLBACK(gnac_profiles_wavpack_generate_pipeline), "slider-extra-processing",
    "//variable[@id='extra-processing']/*");
  widget2 = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder, "label-extra-processing")),
  gnac_profiles_utils_add_description_tooltip(&wavpack_bfi,
    "//variable[@id='extra-processing']/description", widget, widget2, NULL);

  //Joint stereo mode
  widget = gnac_profiles_utils_all_in_one_for_combo(&wavpack_bfi, 
    G_CALLBACK(gnac_profiles_wavpack_generate_pipeline), "combo-joint-stereo-mode",
    "//variable[@id='joint-stereo-mode']/*");
  gnac_profiles_utils_add_toggle_signal(&wavpack_bfi, "checkbutton-joint-stereo-mode",
    G_CALLBACK(gnac_profiles_wavpack_joint_stereo_mode_on_toggle), widget);

  //MD5
  widget = gnac_profiles_utils_all_in_one_for_check(&wavpack_bfi,
    G_CALLBACK(gnac_profiles_wavpack_generate_pipeline), "checkbutton-md5",
    "//variable[@id='md5']/*");
  gnac_profiles_utils_add_description_tooltip(&wavpack_bfi,
    "//variable[@id='md5']/description", widget, NULL);

 //Bitrate control
  widget = gnac_profiles_utils_all_in_one_for_combo(&wavpack_bfi, 
    G_CALLBACK(gnac_profiles_wavpack_generate_pipeline), "combo-bitrate-control",
    "//variable[@id='bitrate-control']/*");
  widget2 = gnac_profiles_utils_add_toggle_signal(&wavpack_bfi, "checkbutton-bitrate-control",
    G_CALLBACK(gnac_profiles_wavpack_bitrate_control_on_toggle), widget);
  gnac_profiles_wavpack_bitrate_control_on_toggle(GTK_TOGGLE_BUTTON(widget2), widget);

  gnac_profiles_xml_engine_free_doc_xpath(wavpack_bfi.doc);
  wavpack_bfi.doc = NULL;

  return wavpack_bfi.format_id;
}

static void
gnac_profiles_wavpack_bitrate_control_on_changed(GtkComboBox *widget)
{
  gint index;
  gint i;
  GtkWidget *check;
  
  GtkWidget *widgets_tab[4] = {
    GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder, "label-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder, "slider-bitrate")),
    GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder, "label-bits-per-sample")),
    GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder, "slider-bits-per-sample")),
  };
  
  check = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder,
    "checkbutton-bitrate-control"));
  index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
  
  for (i = 0; i<4; ++i) {
    if (((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check))))
        &&
        (   ((index == 0) && (i < 2))
        ||  ((index == 1) && (i >= 2)))) {
      gtk_widget_show(widgets_tab[i]);
    } else {
      gtk_widget_hide(widgets_tab[i]);
    }
  }
}


void
gnac_profiles_wavpack_generate_pipeline(GtkWidget *w)
{
  GtkWidget *widget; 
  GtkWidget *check;
  gchar *pipeline;
  gint index;

  pipeline = gnac_profiles_default_generate_pipeline(&wavpack_bfi);
  
  widget = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder, "combo-bitrate-control"));

  if (w == widget) {
    gnac_profiles_wavpack_bitrate_control_on_changed(GTK_COMBO_BOX(w));
  }

  pipeline = gnac_profiles_utils_add_properties_combo(pipeline, &wavpack_bfi, 
    "combo-mode",  NULL); 
  pipeline = gnac_profiles_utils_add_properties_slider(pipeline, &wavpack_bfi, 
    "slider-extra-processing", NULL); 
  pipeline = gnac_profiles_utils_add_properties_checked_combo(pipeline, &wavpack_bfi,
    "combo-joint-stereo-mode", "checkbutton-joint-stereo-mode", NULL);
  pipeline = gnac_profiles_utils_add_properties_check(pipeline, &wavpack_bfi,
    "checkbutton-md5", NULL);

  check = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder, "checkbutton-bitrate-control"));
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check))) {
    index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
    if (index == 0) {
      widget = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder, "slider-bitrate"));
      pipeline = gnac_profiles_utils_add_property_slider(pipeline, "%.0f",
        1000, widget); 
    } else {
      pipeline = gnac_profiles_utils_add_properties_slider(pipeline, &wavpack_bfi, 
       "slider-bits-per-sample", NULL); 
    }
  }

  update_text_buffer_call_back(pipeline);

  if (wavpack_bfi.pipeline != NULL) g_free(wavpack_bfi.pipeline);
  wavpack_bfi.pipeline = pipeline;

}

static void
gnac_profiles_wavpack_bitrate_control_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data)
{
  GtkWidget *label;
  GtkWidget *hbox;
  
  label = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder,
    "label-control-method"));
  hbox = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder,
    "hbox-control-method"));
  if (gtk_toggle_button_get_active(togglebutton)) {
    gtk_widget_show(hbox);
    gtk_widget_show(label);
  } else {
    gtk_widget_hide(hbox);
    gtk_widget_hide(label);
  }
  gnac_profiles_wavpack_bitrate_control_on_changed(GTK_COMBO_BOX(user_data));
  gnac_profiles_wavpack_generate_pipeline(GTK_WIDGET(togglebutton));
}

static void
gnac_profiles_wavpack_joint_stereo_mode_on_toggle(GtkToggleButton *togglebutton,
  gpointer user_data)
{
  gnac_profiles_utils_on_toggle_optionnal_property(togglebutton,user_data);
  gnac_profiles_wavpack_generate_pipeline(GTK_WIDGET(togglebutton));
}

void
gnac_profiles_wavpack_fullfill_fields(AudioProfileWavpack *profile)
{
  GtkWidget *widget;
  GtkWidget *check;

  if (profile == NULL) {
    gnac_profiles_default_fullfill_fields(NULL, &wavpack_bfi);
    gnac_profiles_utils_reinit_values_combo(&wavpack_bfi,
      "combo-mode","combo-bitrate-control", "combo-joint-stereo-mode", NULL);
    gnac_profiles_utils_reinit_values_check(&wavpack_bfi,
      "checkbutton-md5", NULL);
    gnac_profiles_utils_reinit_values_slider(&wavpack_bfi,
      "slider-bitrate", "slider-bits-per-sample", "slider-extra-processing", NULL);
   
    gnac_profiles_utils_set_active_toggle_button(&wavpack_bfi, FALSE,
      "checkbutton-bitrate-control", NULL);
      
  } else {
    gnac_profiles_utils_set_values_combo(&wavpack_bfi,
      "combo-mode", profile->mode, 
      NULL);
    gnac_profiles_utils_set_values_checked_combo(&wavpack_bfi,
      "combo-joint-stereo-mode", "checkbutton-joint-stereo-mode", profile->joint_stereo_mode,
      NULL);
    gnac_profiles_utils_set_values_slider(&wavpack_bfi,"slider-extra-processing",
      profile->extra_processing, NULL);
    gnac_profiles_utils_set_values_check(&wavpack_bfi, "checkbutton-md5",
      profile->md5, NULL);

    widget = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder,
      "combo-bitrate-control"));
    check = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder,
      "checkbutton-bitrate-control"));
    if (profile->bitrate != -1.0) {
      gnac_profiles_utils_set_values_slider(&wavpack_bfi,"slider-bitrate",
        profile->bitrate, NULL);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), TRUE);
      gtk_combo_box_set_active(GTK_COMBO_BOX(widget), 0);
    } else if (profile->bits_per_sample != -1) {
      gnac_profiles_utils_set_values_slider(&wavpack_bfi,"slider-bits-per-sample",
        profile->bits_per_sample, NULL);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), TRUE);
      gtk_combo_box_set_active(GTK_COMBO_BOX(widget), 1);
    }
  }
}


gchar *
gnac_profiles_wavpack_get_combo_format_name()
{
  return gnac_profiles_default_get_combo_format_name(&wavpack_bfi);
}

static AudioProfileWavpack *
gnac_profiles_wavpack_allocate_audio_profile(void)
{
  AudioProfileWavpack *profile;

  profile = g_malloc(sizeof(AudioProfileWavpack));
  
  profile->mode = NULL;
  profile->bitrate = -1;
  profile->bits_per_sample = -1;
  profile->extra_processing = -1;
  profile->joint_stereo_mode = NULL;
  profile->md5 = NULL;

  return profile;
}

void
gnac_profiles_wavpack_free_audio_profile(AudioProfileWavpack *profile)
{
  gnac_profiles_utils_free_audio_profile_generic(profile->generic); 
  
  if (profile->mode != NULL) g_free(profile->mode);
  if (profile->joint_stereo_mode != NULL) g_free(profile->joint_stereo_mode);
  if (profile->md5 != NULL) g_free(profile->md5);
  g_free(profile);
}


AudioProfileWavpack *
gnac_profiles_wavpack_generate_audio_profile(GError **error)
{
  AudioProfileGeneric *generic;
  AudioProfileWavpack *profile;
  GtkWidget *widget;
  GtkWidget *check;
  gint index;

  generic = gnac_profiles_default_generate_audio_profile(&wavpack_bfi);

  profile = gnac_profiles_wavpack_allocate_audio_profile();
  profile->generic = generic;


  gnac_profiles_utils_get_values_combo_and_set(&wavpack_bfi,
    "combo-mode", &profile->mode,
    NULL);
  gnac_profiles_utils_get_values_checked_combo_and_set(&wavpack_bfi,
    "combo-joint-stereo-mode", "checkbutton-joint-stereo-mode", &profile->joint_stereo_mode,
    NULL);
  gnac_profiles_utils_get_values_slider_and_set(&wavpack_bfi,
    "slider-extra-processing", &profile->extra_processing,NULL);

  gnac_profiles_utils_get_values_check_and_set(&wavpack_bfi,
    "checkbutton-md5", &profile->md5, NULL);

  widget = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder,
    "combo-bitrate-control"));
  check = GTK_WIDGET(gtk_builder_get_object(wavpack_bfi.builder,
    "checkbutton-bitrate-control"));

  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check))) {
    index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
    if (index == 0) {
      gnac_profiles_utils_get_values_slider_and_set(&wavpack_bfi,
        "slider-bitrate", &profile->bitrate, NULL);
    } else {
      gnac_profiles_utils_get_values_slider_and_set(&wavpack_bfi,
        "slider-bits-per-sample", &profile->bits_per_sample, NULL);
    }
  }
  return profile;
}


GtkWidget *
gnac_profiles_wavpack_get_widget(AudioProfileWavpack *profile)
{
  if (profile != NULL) {
    gnac_profiles_wavpack_fullfill_fields(profile);
  }
  return gnac_profiles_default_get_widget(profile,&wavpack_bfi);
}

void
gnac_profiles_wavpack_save_profile(AudioProfileWavpack *profile)
{
  XMLDoc *doc;
  gchar *bitrate_str = NULL;
  gchar *bits_per_sample_str = NULL;
  gchar *extra_processing_str;
  
  if (profile->bitrate != -1) {
    bitrate_str = gnac_profiles_utils_gdouble_to_gchararray(
      profile->bitrate);
  } else if (profile->bits_per_sample != -1) {
    bits_per_sample_str = gnac_profiles_utils_gdouble_to_gchararray(
      profile->bits_per_sample);
  }
  extra_processing_str = gnac_profiles_utils_gdouble_to_gchararray(
    profile->extra_processing);

  doc = gnac_profiles_default_save_profile(profile->generic, &wavpack_bfi);
  gnac_profiles_utils_save_specific_properties_in_file(doc, 
    "mode", profile->mode,
    "bitrate", (bitrate_str == NULL) ? "" : bitrate_str,
    "bits-per-sample", (bits_per_sample_str == NULL) ? "" : bits_per_sample_str,
    "extra-processing", extra_processing_str,
    "joint-stereo-mode", (profile->joint_stereo_mode == NULL) ? "" : profile->joint_stereo_mode ,
    "md5", profile->md5,
    NULL
    ); 
  gnac_profiles_xml_engine_save_doc(doc, profile->generic->name);
  gnac_profiles_xml_engine_free_doc_xpath(doc);
  if (bitrate_str != NULL) g_free(bitrate_str);
  if (bits_per_sample_str != NULL) g_free(bits_per_sample_str);
  g_free(extra_processing_str);
}


AudioProfileWavpack *
gnac_profiles_wavpack_load_specific_properties(XMLDoc *doc, AudioProfileGeneric *generic)
{
  AudioProfileWavpack *profile;
  gchar *bitrate_str;
  gchar *bits_per_sample_str;
  gchar *extra_processing_str;
  
  profile = gnac_profiles_wavpack_allocate_audio_profile();
  profile->generic = generic;
  gnac_profiles_utils_load_saved_profile(doc, "/audio-profile/format-specific/",
    "mode", &profile->mode,
    "bitrate", &bitrate_str,
    "bits-per-sample", &bits_per_sample_str,
    "extra-processing", &extra_processing_str,
    "joint-stereo-mode", &profile->joint_stereo_mode,
    "md5", &profile->md5,
    NULL
    );

  profile->extra_processing = gnac_profiles_utils_gchararray_to_gdouble(
    extra_processing_str);
  g_free(extra_processing_str);
  if (bitrate_str != NULL) {
    profile->bitrate = gnac_profiles_utils_gchararray_to_gdouble(bitrate_str);
    g_free(bitrate_str);
  } else if (bits_per_sample_str != NULL) {
    profile->bits_per_sample = gnac_profiles_utils_gchararray_to_gdouble(
      bits_per_sample_str);
    g_free(bits_per_sample_str);
  }

  return profile;
}

void
gnac_profiles_wavpack_clean_up(void) {
  gnac_profiles_default_clean_up(&wavpack_bfi);
  gnac_profiles_utils_free_combo_values(&wavpack_bfi, "combo-bitrate-control",
    "combo-mode", "combo-joint-stereo-mode", NULL);
  gnac_profiles_utils_free_check_values(&wavpack_bfi, "checkbutton-md5",
    NULL);
  gnac_profiles_utils_free_slider_values(&wavpack_bfi, "slider-bitrate",
    "slider-bits-per-sample", "slider-extra-processing", NULL);
  gnac_profiles_utils_free_basic_format_info_content(&wavpack_bfi);
}

const gchar *
gnac_profiles_wavpack_get_plugin_name(void)
{
  return wavpack_bfi.format_plugin_name;
}

const gchar *
gnac_profiles_wavpack_get_description(void)
{
  return wavpack_bfi.description;
}
