/*
 * $Id: gnac-profiles-default.c 239 2009-03-11 10:09:00Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include "gnac-profiles-default.h"
#include "libgnac-debug.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void
gnac_profiles_default_init(BasicFormatInfo *bfi, GCallback pipeline_callback)
{
  GtkWidget *widget;
  XMLDoc *base_values_doc;
  GtkWidget *profile_advanced_base_widget;
  GError *error = NULL;

  bfi->builder = gtk_builder_new();

  if (!gtk_builder_add_from_file(bfi->builder, bfi->gtkbuilder_xml_file, &error)) {
    libgnac_critical("%s\n", error->message);
    g_clear_error(&error);
    exit(EXIT_FAILURE);
  }

  widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, "properties-alignment"));
  if (!gtk_builder_add_from_file(bfi->builder, 
      PKGDATADIR "/profiles/gnac-profiles-base-advanced.xml", &error)) {
    libgnac_critical("%s\n", error->message);
    g_error_free(error);
    exit(EXIT_FAILURE);
  }

  profile_advanced_base_widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder,
    "table-advanced-base"));
  
  widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, "properties-table-advanced"));
  gtk_table_attach_defaults(GTK_TABLE(widget),profile_advanced_base_widget,0,2,0,1);
  
  //Init values
  bfi->doc = gnac_profiles_xml_engine_load_doc_xpath(bfi->data_xml_file);
  base_values_doc = gnac_profiles_xml_engine_load_doc_xpath(PKGDATADIR "/profiles/base.xml");
  
  //format id
  bfi->format_id = gnac_profiles_xml_engine_get_format_id(bfi->doc);
  //name and extension
  gnac_profiles_utils_init_name_extension(bfi->doc,bfi->format_id,
    &(bfi->format_name), &(bfi->file_extension));

  //description text
  bfi->description = gnac_profiles_utils_init_description(bfi->doc, bfi->format_id);

  //format plugin name
  bfi->format_plugin_name = gnac_profiles_xml_engine_get_text_node(bfi->doc,
    "//gst-plugin-name");

  //Advanced
  //channels
  widget = gnac_profiles_utils_register_advanced_channels(bfi->builder,
    base_values_doc, pipeline_callback, bfi);
  
  //sample rate
  widget = gnac_profiles_utils_register_advanced_sample_rate(bfi->builder,
    base_values_doc, pipeline_callback, bfi);

  gnac_profiles_utils_init_raw_audioconvert(base_values_doc);
  
  //pipeline
  //encoder
  bfi->pipeline_encoder = gnac_profiles_xml_engine_get_text_node(bfi->doc,
    "//process[@id='gstreamer-audio']");
  
  //multiplexers
  bfi->pipeline_multiplexers = g_strconcat("",NULL);
  GList *multiplexers_list = gnac_profiles_xml_engine_get_list_values(bfi->doc,
    "//process[@id='multiplexer']/value");
  if (multiplexers_list != NULL) {
    bfi->pipeline_multiplexers= gnac_profiles_utils_add_pipes(
      bfi->pipeline_multiplexers, multiplexers_list);

    g_list_free(multiplexers_list);
  }

  gnac_profiles_xml_engine_free_doc_xpath(base_values_doc);
}

GtkWidget *
gnac_profiles_default_get_widget(void *profile, BasicFormatInfo *bfi)
{
  GtkWidget *widget; 

  widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, "properties-alignment"));
  return widget;
}

gchar *
gnac_profiles_default_generate_pipeline(BasicFormatInfo *bfi)
{
  GtkWidget *widget;
  gchar *channels;
  gchar *rate;
  gchar *pipeline;
  
  widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, "combo-channels"));
  channels = gnac_profiles_utils_get_value_combo(widget);
  
  widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, "combo-sample-rate"));
  rate = gnac_profiles_utils_get_value_combo(widget);

  pipeline = gnac_profiles_utils_get_basepipeline(channels,rate);
  g_free(channels);
  g_free(rate);

  pipeline = gnac_profiles_utils_add_pipe(pipeline,bfi->pipeline_encoder);
  
  return pipeline;

}

gchar *
gnac_profiles_default_get_combo_format_name(BasicFormatInfo *bfi)
{
  return gnac_profiles_utils_get_combo_format_name(bfi->format_name,bfi->file_extension);
}


void
gnac_profiles_default_fullfill_fields(AudioProfileGeneric *profile,
  BasicFormatInfo *bfi)
{
  GtkWidget *widget;
  
  if (profile == NULL) {
    gnac_profiles_utils_reinit_values_combo(bfi, 
      "combo-channels","combo-sample-rate", NULL);
  } else {
  
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, "combo-channels"));
    gnac_profiles_utils_set_value_combo(widget, profile->channels);

    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, "combo-sample-rate"));
    gnac_profiles_utils_set_value_combo(widget, profile->rate);
  }
}

AudioProfileGeneric *
gnac_profiles_default_generate_audio_profile(BasicFormatInfo *bfi)
{
  AudioProfileGeneric *profile;

  profile = gnac_profiles_utils_allocate_audio_profile_generic();
  profile->extension = g_strdup(bfi->file_extension);
  profile->format_id = g_strdup(bfi->format_id);
  profile->format_name = g_strdup(bfi->format_name);
  profile->pipeline = g_strdup(bfi->pipeline);

  profile->channels = gnac_profiles_utils_get_value_combo(
    GTK_WIDGET(gtk_builder_get_object(bfi->builder,"combo-channels")));

  profile->rate = gnac_profiles_utils_get_value_combo(
    GTK_WIDGET(gtk_builder_get_object(bfi->builder,"combo-sample-rate")));

  return profile;
}

XMLDoc *
gnac_profiles_default_save_profile(AudioProfileGeneric *profile,
BasicFormatInfo *bfi)
{
  XMLDoc *doc;
  AudioProfileGeneric *generic;

  generic = profile->generic;
  doc = gnac_profiles_xml_engine_load_doc_xpath
    (PKGDATADIR "/profiles/profile-base-save.xml");
  gnac_profiles_xml_engine_modify_values(doc, "/audio-profile/base/*", 
    generic->pipeline, generic->rate, generic->channels, generic->description,
    generic->name, generic->extension, generic->format_name, generic->format_id,
    NULL);
  
  return doc;
}

XMLDoc *
gnac_profiles_default_load_generic_audio_profile(const gchar *filename, AudioProfileGeneric **generic)
{
  XMLDoc *doc;
  AudioProfileGeneric *profile;

  doc = gnac_profiles_xml_engine_load_doc_xpath(filename);
  if (doc != NULL) {
    profile = gnac_profiles_utils_allocate_audio_profile_generic();
    *generic = profile;
    gnac_profiles_utils_load_saved_profile(doc, "/audio-profile/base/",
      "format-id", &profile->format_id,
      "format-name", &profile->format_name,
      "format-extension", &profile->extension,
      "name", &profile->name,
      "description", &profile->description,
      "channels", &profile->channels,
      "sample-rate", &profile->rate,
      "pipeline", &profile->pipeline,
      NULL);
  } else {
    *generic = NULL;
  }
  
  return doc;
}

void
gnac_profiles_default_clean_up(BasicFormatInfo *bfi)
{
  GtkWidget *widget_advanced;
  GtkWidget *widget;

  widget_advanced = GTK_WIDGET(gtk_builder_get_object(bfi->builder,
    "table-advanced-base"));
  widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder,
    "properties-alignment"));
  gnac_profiles_utils_free_combo_values(bfi, "combo-channels", 
    "combo-sample-rate", NULL);
  gtk_widget_destroy(widget_advanced);
  gtk_widget_destroy(widget);
}
