/*
 * $Id: gnac-profiles-utils.c 239 2009-03-11 10:09:00Z bdupasqu $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include "gnac-profiles-utils.h"
#include "libgnac-debug.h"

#include "stdlib.h"

gchar *raw = NULL;
gchar *audioconvert = NULL;

void
gnac_profiles_utils_init_name_extension(XMLDoc *doc, const gchar *format, 
  gchar **name, gchar **extension)
{
  gchar *xpath_query_name;
  gchar *xpath_query_extension;

  xpath_query_name = g_strconcat("//profile[@id='", format, "']/name", NULL);
  xpath_query_extension = g_strconcat("//profile[@id='", format,
    "']/output-file-extension", NULL);
  *name = gnac_profiles_xml_engine_get_text_node(doc, xpath_query_name);
  *extension = gnac_profiles_xml_engine_get_text_node(doc, xpath_query_extension);
  
  g_free(xpath_query_name);
  g_free(xpath_query_extension);
}


gchar *
gnac_profiles_utils_get_combo_format_name(gchar *name, gchar *extension)
{
  return g_strconcat(name, " (.", extension, ")", NULL);
}

gchar *
gnac_profiles_utils_init_description(XMLDoc *doc, const gchar *format)
{
  gchar *xpath_query;
  gchar *raw_desc;
  gchar *desc;

  xpath_query = g_strconcat("//profile[@id='", format, "']/description", NULL);
  raw_desc = gnac_profiles_xml_engine_get_text_node(doc,xpath_query);
  desc = g_strconcat("<i>", raw_desc, "</i>", NULL);
  g_free(xpath_query);
  g_free(raw_desc);

  return desc;
}

GtkWidget *
gnac_profiles_utils_all_in_one_for_combo(BasicFormatInfo *bfi,
  GCallback call_back, const gchar *combo_name, const gchar *xpath_query)
{
  GtkWidget* widget;
  
  widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, combo_name));
  if (widget != NULL) {
    gnac_profiles_utils_register_combo(widget,bfi->doc,xpath_query,
      call_back,NULL); 
  } else {
    libgnac_warning("Combo %s not found\n", combo_name);
  }
  return widget;
}

GtkWidget *
gnac_profiles_utils_all_in_one_for_slider(BasicFormatInfo *bfi,
  GCallback call_back, const gchar *slider_name, const gchar *xpath_query)
{
  GtkWidget* widget;
  
  widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, slider_name));
  if (widget != NULL) {
    gnac_profiles_utils_register_slider(widget,bfi->doc,xpath_query,
      call_back,NULL); 
  } else {
    libgnac_warning("Slider %s not found\n", slider_name);
  }
  return widget;
}

GtkWidget *
gnac_profiles_utils_all_in_one_for_check(BasicFormatInfo *bfi,
  GCallback call_back, const gchar *check_name, const gchar *xpath_query)
{
  GtkWidget* widget;
  
  widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, check_name));
  if (widget != NULL) {
    gnac_profiles_utils_register_check(widget,bfi->doc,xpath_query,
      call_back,NULL);
  } else {
    libgnac_warning("Check %s not found\n", check_name);
  }
  return widget;
}


GtkWidget *
gnac_profiles_utils_add_toggle_signal(BasicFormatInfo *bfi,
  const gchar *check, GCallback f, gpointer user_data)
{
  GtkWidget *widget;

  widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder,
    check));
  g_signal_connect (G_OBJECT (widget), "toggled", f, user_data);

  return widget;
}

gint
gnac_profiles_utils_register_combo(GtkWidget *widget, XMLDoc *doc, 
  const gchar *xpath_query, GCallback call_back, gpointer user_data)
{
  ComboValues *combo;
  gint result;

  combo = g_malloc(sizeof(ComboValues));
  result = gnac_profiles_xml_engine_get_combo_values(doc,xpath_query,combo);
  if (result == 0) {
    gnac_profiles_utils_add_values_combo(widget,combo);
    g_object_set_data(G_OBJECT(widget), "combo-values", combo);
    g_signal_connect (G_OBJECT(widget), "changed", call_back , user_data);
  } else {
    libgnac_warning("Impossible to register combo with xpath query %s\n", xpath_query);
    g_free(combo);
  }
  return result;
}

gint
gnac_profiles_utils_register_slider(GtkWidget *widget, XMLDoc *doc, 
  const gchar *xpath_query, GCallback call_back, gpointer user_data)
{
  SliderValues *slider;
  gint result;

  slider = g_malloc(sizeof(SliderValues));
  result = gnac_profiles_xml_engine_get_slider_values(doc,xpath_query,slider);
  if (result == 0) {
    gnac_profiles_utils_add_values_slider(widget,slider);
    g_object_set_data(G_OBJECT(widget), "slider-values", slider);
    g_signal_connect (G_OBJECT(widget), "value-changed", call_back , user_data);
    //g_signal_connect (G_OBJECT(widget), "button-release-event", call_back , user_data);
    //g_signal_connect (G_OBJECT(widget), "key-release-event", call_back , user_data);
    //g_signal_connect (G_OBJECT(widget), "leave-notify-event", call_back , user_data);
  } else {
    libgnac_warning("Impossible to register slider with xpath query %s\n", xpath_query);
    g_free(slider);
  }
  return result;
}


gint
gnac_profiles_utils_register_check(GtkWidget *widget, XMLDoc *doc, 
  const gchar *xpath_query, GCallback call_back, gpointer user_data)
{
  CheckValues *check;
  gint result;

  check = g_malloc(sizeof(CheckValues));
  result = gnac_profiles_xml_engine_get_check_values(doc, xpath_query, check);
  if (result == 0) {
    g_object_set_data(G_OBJECT(widget), "check-values", check);
    g_signal_connect (G_OBJECT(widget), "toggled", call_back , user_data);
  } else {
    libgnac_warning("Impossible to register check with xpath query %s\n", xpath_query);
    g_free(check);
  }
  return result;
}


GtkWidget *
gnac_profiles_utils_register_advanced_channels(GtkBuilder *builder, 
  XMLDoc *doc, GCallback call_back, gpointer user_data)
{
  GtkWidget *widget = NULL;

  widget = GTK_WIDGET(gtk_builder_get_object(builder, "combo-channels"));
  if (widget != NULL) {
    gnac_profiles_utils_register_combo(widget, doc,
      "//variable[@id='channels']/*", call_back, user_data);
  } else {
    libgnac_warning("Impossible to find widget %s\n", "combo-channels");
  }
  return widget;
}

GtkWidget *
gnac_profiles_utils_register_advanced_sample_rate(GtkBuilder *builder, 
  XMLDoc *doc, GCallback call_back, gpointer user_data)
{
  GtkWidget *widget = NULL;

  widget = GTK_WIDGET(gtk_builder_get_object(builder, "combo-sample-rate"));
  if (widget != NULL) {
    gnac_profiles_utils_register_combo(widget, doc,
      "//variable[@id='sample-rate']/*", call_back, user_data);
  } else {
    libgnac_warning("Impossible to find widget %s\n", "combo-sample-rate");
  }
  return widget;
}


void
gnac_profiles_utils_add_values_combo(GtkWidget *combo, ComboValues *values)
{
  GtkTreeModel *model;
  GtkCellRenderer *renderer;
  GList *list_names;
  GList *list_values;

  model = GTK_TREE_MODEL(gtk_list_store_new(1,G_TYPE_STRING));
  renderer = gtk_cell_renderer_text_new();
  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), 
    renderer, FALSE);
  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo), 
    renderer, "text", 0, NULL);
  gtk_combo_box_set_model(GTK_COMBO_BOX(combo),model);

  list_names = values->names;
  list_values= values->values;
  gint index = 0;
  while (list_names != NULL) {
    gtk_combo_box_append_text(GTK_COMBO_BOX(combo),
      (const gchar*) list_names->data);
    
    if (!g_strcmp0(values->default_value,
        (const gchar*) list_values->data)) {
      gtk_combo_box_set_active(GTK_COMBO_BOX(combo),index);
    }

    list_names = list_names->next;
    list_values = list_values->next;
    ++index;
  }
}

void
gnac_profiles_utils_reinit_values_combo(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  ComboValues *values;
  va_list ap;
  void *v;

  va_start(ap,bfi);
  v = va_arg(ap,void*);
  while (v != NULL) {
    const gchar *combo_name = (const gchar *) v;
    
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, combo_name));
    values = g_object_get_data(G_OBJECT(widget), "combo-values");
    
    gnac_profiles_utils_set_value_combo(widget, values->default_value);

    v = va_arg(ap,void*);
  }
  va_end(ap);
}

void
gnac_profiles_utils_set_value_combo(GtkWidget *combo, const gchar *value)
{
  ComboValues *values;
  GList *list_values;
  gint index;
  
  values = g_object_get_data(G_OBJECT(combo), "combo-values");
  list_values= values->values;
  if (value == NULL) value = values->default_value;
  index = 0;
  while (list_values != NULL) {
    if (!g_strcmp0(value, (const gchar*) list_values->data)) {
      gtk_combo_box_set_active(GTK_COMBO_BOX(combo),index);
    }
    list_values = list_values->next;
    ++index;
  }
}

void
gnac_profiles_utils_set_values_combo(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  const gchar *value;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {
    
    value = va_arg(ap,const gchar *);

    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    gnac_profiles_utils_set_value_combo(widget, value);

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
}

void
gnac_profiles_utils_set_values_checked_combo(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name_combo;
  const gchar *name_checkbox;
  const gchar *value;

  va_start(ap,bfi);
  name_combo = va_arg(ap,const gchar *);
  while (name_combo != NULL) {
    
    name_checkbox = va_arg(ap,const gchar *);
    value = va_arg(ap,const gchar *);
    
    if (value != NULL) {
      widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name_combo));
      gnac_profiles_utils_set_value_combo(widget, value);
    }
    
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name_checkbox));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), (value != NULL));

    name_combo = va_arg(ap, const gchar *);
  }
  va_end(ap);
}


gchar *
gnac_profiles_utils_get_value_combo(GtkWidget *combo)
{
  ComboValues *values;
  gint index;
  
  index =  gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
  values = g_object_get_data(G_OBJECT(combo),"combo-values");
  return g_strconcat(g_list_nth_data(values->values,index),NULL);
}

void
gnac_profiles_utils_add_values_slider(GtkWidget *slider, SliderValues *values)
{
  gtk_scale_set_digits(GTK_SCALE(slider), 0);
  gtk_range_set_range(GTK_RANGE(slider),values->min,values->max);
  gtk_range_set_value(GTK_RANGE(slider),values->default_value);
  gtk_range_set_increments(GTK_RANGE(slider), values->step, values->step);
}

void
gnac_profiles_utils_get_values_combo_and_set(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  gchar **value;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {
    
    value = va_arg(ap,gchar **);

    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    *value = gnac_profiles_utils_get_value_combo(widget);

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);

}

void
gnac_profiles_utils_get_values_checked_combo_and_set(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  const gchar *checkbox_name;
  const gchar **value;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {
    checkbox_name = va_arg(ap, const gchar *);
    value = va_arg(ap,const gchar **);

    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, checkbox_name));
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
      widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
      *value = gnac_profiles_utils_get_value_combo(widget);
    }

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
}

void
gnac_profiles_utils_reinit_values_slider(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  SliderValues *values;
  va_list ap;
  void *v;

  va_start(ap,bfi);
  v = va_arg(ap,void*);
  while (v != NULL) {
    const gchar *slider_name = (const gchar *) v;
    
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, slider_name));
    values = g_object_get_data(G_OBJECT(widget), "slider-values");
    
    gtk_range_set_value(GTK_RANGE(widget),values->default_value);

    v = va_arg(ap,void*);
  }
  va_end(ap);
}

void
gnac_profiles_utils_set_values_slider(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  gdouble value;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {

    value = va_arg(ap,gdouble);
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    gtk_range_set_value(GTK_RANGE(widget),value);

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
}

void
gnac_profiles_utils_set_values_checked_slider(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  const gchar *checkbox_name;
  gdouble value;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {

    checkbox_name = va_arg(ap, const gchar *); 
    value = va_arg(ap,gdouble);

    if (value != -1) {
      widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
      gtk_range_set_value(GTK_RANGE(widget),value);
    }
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, checkbox_name));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), (value != -1));

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
  
}

void
gnac_profiles_utils_get_values_slider_and_set(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  gdouble *value;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL){
    
    value = va_arg(ap,gdouble*);

    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    *value = gtk_range_get_value(GTK_RANGE(widget));

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
}

void
gnac_profiles_utils_get_values_checked_slider_and_set(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  const gchar *checkbox_name;
  gdouble *value;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {

    checkbox_name = va_arg(ap, const gchar *); 
    value = va_arg(ap,gdouble*);
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, checkbox_name));
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
      widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
      *value = gtk_range_get_value(GTK_RANGE(widget));
    }

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);

}

void
gnac_profiles_utils_reinit_values_check(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  CheckValues *values;
  va_list ap;
  void *v;

  va_start(ap,bfi);
  v = va_arg(ap,void*);
  while (v != NULL) {
    const gchar *check_name = (const gchar *) v;
    
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, check_name));
    values = g_object_get_data(G_OBJECT(widget), "check-values");
    
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
      values->default_value);

    v = va_arg(ap,void*);
  }
  va_end(ap);
}

void
gnac_profiles_utils_set_values_check(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  const gchar *value;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {

    value = va_arg(ap,const gchar *);
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
      (!g_strcmp0(value, "true")));

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
}

void
gnac_profiles_utils_get_values_check_and_set(BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  const gchar **value;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL){
    
    value = va_arg(ap,const gchar **);
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    *value = g_strdup((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))
                        ? "true" : "false"));
                      
    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
}

void
gnac_profiles_utils_add_description_tooltip(BasicFormatInfo *bfi,
  const gchar *xpath_query, ...)
{
  GtkWidget *widget;
  gchar *description;
  va_list ap;
  
  description = gnac_profiles_xml_engine_get_text_node(bfi->doc, xpath_query);
  if (description != NULL) {
    va_start(ap, xpath_query);
    widget = va_arg(ap, GtkWidget*);
    while (widget != NULL) {
      gtk_widget_set_tooltip_markup(widget, description);
      widget = va_arg(ap, GtkWidget*);
    }
    g_free(description);
  } else {
    libgnac_warning("Impossible to find description in %s\n", xpath_query);
  }
}

void
gnac_profiles_utils_init_raw_audioconvert(XMLDoc *doc)
{
  if ((raw == NULL) && (audioconvert == NULL)) {
    raw = gnac_profiles_xml_engine_get_text_node(doc,
      "//process[@id='gstreamer-audio']");

    audioconvert = gnac_profiles_xml_engine_get_text_node(doc,
      "//process[@id='audioconvert']");
  }
}

gchar *
gnac_profiles_utils_get_basepipeline(const gchar *channels, const gchar *rate)
{
  return g_strconcat(audioconvert," ! ", raw, ", rate=",rate, ", channels=",
    channels, " ! ", audioconvert, NULL);
}

gchar *
gnac_profiles_utils_add_pipe(gchar *pipeline, gchar *new_pipe)
{
  gchar *temp;
  temp = g_strconcat(pipeline, " ! ", new_pipe, NULL);
  g_free(pipeline);
  return temp;
}

gchar *
gnac_profiles_utils_add_pipes(gchar *pipeline, GList *values)
{
  GList *list_temp = values;
  while (list_temp != NULL) {
    Value *v = list_temp->data;
    gchar *temp;
    if (!g_strcmp0("",pipeline)) {
      temp = g_strconcat(v->value,NULL);
    } else {
      temp = g_strconcat(pipeline," ! ",v->value,NULL);
    }
    g_free(pipeline);
    g_free(v->name);
    g_free(v->value);
    g_free(v);
    pipeline = temp;
    list_temp = list_temp->next;
  }
  return pipeline;;
}

gchar *
gnac_profiles_utils_add_property_combo(gchar *pipeline, GtkWidget *combo)
{
  gchar *temp;
  ComboValues *values;
  gint index;

  index =  gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
  values = g_object_get_data(G_OBJECT(combo),"combo-values");
  gchar *value = g_list_nth_data(values->values,index);
  temp = g_strconcat(pipeline," ", values->variable_name, "=", value ,NULL);
  g_free(pipeline);
  return temp;
}

gchar *
gnac_profiles_utils_add_properties_combo(gchar *pipeline, BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {
    
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    pipeline = gnac_profiles_utils_add_property_combo(pipeline, widget);

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);

  return pipeline;
}

gchar *
gnac_profiles_utils_add_properties_checked_combo(gchar *pipeline, BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  const gchar *checkbox_name;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {
    checkbox_name = va_arg(ap, const gchar *);

    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, checkbox_name));
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
      widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
      pipeline = gnac_profiles_utils_add_property_combo(pipeline, widget);
    }

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);

  return pipeline;
}

gchar *
gnac_profiles_utils_add_property_slider(gchar *pipeline, const gchar *format,
  gdouble factor, GtkWidget *slider)
{
  gchar *temp;
  gdouble value;
  gchar *value_str;
  SliderValues *values;
  
  value = gtk_range_get_value(GTK_RANGE(slider));
  values = g_object_get_data(G_OBJECT(slider),"slider-values");
  value_str = gnac_profiles_utils_gdouble_to_gchararray_format(value*factor,format);
  temp = g_strconcat(pipeline," ", values->variable_name, "=",
    value_str, NULL);
  g_free(pipeline);
  g_free(value_str);
  return temp;
}

gchar *
gnac_profiles_utils_add_properties_slider(gchar *pipeline, BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {
    
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    pipeline = gnac_profiles_utils_add_property_slider(pipeline, "%.0f", 1, widget);

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);

  return pipeline;
 
}

gchar *
gnac_profiles_utils_add_properties_slider_format(gchar *pipeline,
  BasicFormatInfo *bfi, const gchar *format, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;

  va_start(ap,format);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {
    
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    pipeline = gnac_profiles_utils_add_property_slider(pipeline, format, 1, widget);

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);

  return pipeline;
}

gchar *
gnac_profiles_utils_add_properties_checked_slider(gchar *pipeline, BasicFormatInfo *bfi, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  const gchar *checkbox_name;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {
    checkbox_name = va_arg(ap, const gchar *);

    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, checkbox_name));
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
      widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
      pipeline = gnac_profiles_utils_add_property_slider(pipeline, "%.0f", 1, widget);
    }

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);

  return pipeline;
}

gchar *
gnac_profiles_utils_add_properties_checked_slider_format(gchar *pipeline,
  BasicFormatInfo *bfi, const gchar *format, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;
  const gchar *checkbox_name;

  va_start(ap,format);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {
    checkbox_name = va_arg(ap, const gchar *);

    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, checkbox_name));
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
      widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
      pipeline = gnac_profiles_utils_add_property_slider(pipeline, format, 1, widget);
    }

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);

  return pipeline;
}

gchar *
gnac_profiles_utils_add_property_check(gchar *pipeline, GtkWidget *check)
{
  gchar *temp;
  gboolean checked;
  CheckValues *values;
  
  checked = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
  values = g_object_get_data(G_OBJECT(check),"check-values");
  temp = g_strconcat(pipeline," ", values->variable_name, "=",
    (checked ? "true" : "false") , NULL);
  g_free(pipeline);
  return temp;
}

gchar *
gnac_profiles_utils_add_properties_check(gchar *pipeline, BasicFormatInfo *bfi,
 ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;

  va_start(ap,bfi);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {

    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    pipeline = gnac_profiles_utils_add_property_check(pipeline, widget);

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);

  return pipeline;
}

void
gnac_profiles_utils_set_active_toggle_button(BasicFormatInfo *bfi, gboolean active, ...)
{
  GtkWidget *widget;
  va_list ap;
  const gchar *name;

  va_start(ap,active);
  name = va_arg(ap,const gchar *);
  while (name != NULL) {
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), active);

    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
}

void
gnac_profiles_utils_on_toggle_optionnal_property(GtkToggleButton *togglebutton,
  gpointer user_data)
{
  GtkWidget *widget = GTK_WIDGET(user_data);
  if (gtk_toggle_button_get_active(togglebutton)) {
    gtk_widget_set_sensitive(widget,TRUE);
  } else {
    gtk_widget_set_sensitive(widget,FALSE);
  }
}

void
gnac_profiles_utils_save_specific_properties_in_file(XMLDoc *doc, ...)
{
  va_list ap;
  
  va_start(ap,doc);
  gnac_profiles_xml_engine_add_values(doc, "/audio-profile/format-specific",
    ap);

}

void
gnac_profiles_utils_load_saved_profile(XMLDoc *doc, const gchar *base_query, 
  ...)
{
  const gchar *spec_query;
  gchar *query;
  gchar **value;
  va_list ap;

  va_start(ap, base_query);
  spec_query = va_arg(ap, const gchar *);
  while (spec_query != NULL) {
    value = va_arg(ap, gchar **);

    query = g_strconcat(base_query, spec_query, NULL);
    *value = gnac_profiles_xml_engine_get_text_node(doc, query);

    g_free(query);
    spec_query = va_arg(ap, const gchar *);
  }
  va_end(ap);
}

AudioProfileGeneric *
gnac_profiles_utils_allocate_audio_profile_generic()
{
  AudioProfileGeneric *profile;

  profile = g_malloc(sizeof(AudioProfileGeneric));
  profile->generic = profile;
  profile->name = NULL;
  profile->description = NULL;
  profile->format_id = NULL;
  profile->format_name = NULL;
  profile->extension = NULL;
  profile->pipeline = NULL;
  profile->rate = NULL;
  profile->channels = NULL;
  
  return profile;
}

void
gnac_profiles_utils_free_audio_profile_generic(AudioProfileGeneric *profile)
{
  if (profile->name != NULL) g_free(profile->name);
  if (profile->description != NULL) g_free(profile->description);
  if (profile->format_id != NULL) g_free(profile->format_id);
  if (profile->format_name != NULL) g_free(profile->format_name);
  if (profile->extension != NULL) g_free(profile->extension);
  if (profile->pipeline != NULL) g_free(profile->pipeline);
  if (profile->rate != NULL) g_free(profile->rate);
  if (profile->channels != NULL) g_free(profile->channels);
  g_free(profile);
}

void
gnac_profiles_utils_free_basic_format_info_content(BasicFormatInfo *bfi)
{
  g_object_unref(bfi->builder);
  if (bfi->doc != NULL) gnac_profiles_xml_engine_free_doc_xpath(bfi->doc);
  if (bfi->pipeline != NULL) g_free(bfi->pipeline);
  if (bfi->pipeline_encoder != NULL) g_free(bfi->pipeline_encoder);
  if (bfi->pipeline_multiplexers != NULL) g_free(bfi->pipeline_multiplexers);
  if (bfi->format_id != NULL) g_free(bfi->format_id);
  if (bfi->format_plugin_name != NULL) g_free(bfi->format_plugin_name);
  if (bfi->format_name != NULL) g_free(bfi->format_name);
  if (bfi->file_extension != NULL) g_free(bfi->file_extension);
  if (bfi->description != NULL) g_free(bfi->description);
}


void
gnac_profiles_utils_free_combo_values(BasicFormatInfo *bfi, ...)
{
  va_list ap;
  GtkWidget *widget;
  const gchar *name;
  ComboValues *values;

  va_start(ap, bfi);
  name = va_arg(ap, const gchar *);
  while (name != NULL) {
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    values = g_object_get_data(G_OBJECT(widget), "combo-values");
    gnac_profiles_xml_engine_free_combo_values(values);
    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
}

void
gnac_profiles_utils_free_slider_values(BasicFormatInfo *bfi, ...)
{
  va_list ap;
  GtkWidget *widget;
  const gchar *name;
  SliderValues *values;

  va_start(ap, bfi);
  name = va_arg(ap, const gchar *);
  while (name != NULL) {
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    values = g_object_get_data(G_OBJECT(widget), "slider-values");
    gnac_profiles_xml_engine_free_slider_values(values);
    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
}

void
gnac_profiles_utils_free_check_values(BasicFormatInfo *bfi, ...)
{
  va_list ap;
  GtkWidget *widget;
  const gchar *name;
  CheckValues *values;

  va_start(ap, bfi);
  name = va_arg(ap, const gchar *);
  while (name != NULL) {
    widget = GTK_WIDGET(gtk_builder_get_object(bfi->builder, name));
    values = g_object_get_data(G_OBJECT(widget), "check-values");
    gnac_profiles_xml_engine_free_check_values(values);
    name = va_arg(ap, const gchar *);
  }
  va_end(ap);
}


gchar *
gnac_profiles_utils_gdouble_to_gchararray(gdouble value)
{
  return gnac_profiles_utils_gdouble_to_gchararray_format(value,"%.0f");
}

gchar *
gnac_profiles_utils_gdouble_to_gchararray_format(gdouble value, const gchar *format)
{
  gchar buffer[64];

  g_ascii_formatd(buffer,64,format,value);
  return g_strdup(buffer);
}

gdouble
gnac_profiles_utils_gchararray_to_gdouble(const gchar * value)
{
  return g_ascii_strtod(value, NULL);
}

void
gnac_profiles_utils_display_error_dialog(GtkWidget *dialog)
{
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_hide(dialog);
  gtk_widget_destroy(dialog);
}
