/*
 * $Id: gnac-profiles-xml-engine.h 240 2009-03-11 20:12:28Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#ifndef GNAC_PROFILES_XML_ENGINE_H
#define GNAC_PROFILES_XML_ENGINE_H

#include <glib.h>

#define GNAC_SAVED_PROFILES_URL(name) saved_profiles_dir , G_DIR_SEPARATOR_S, name
#define GNAC_SAVED_PROFILES_URL_WITH_EXT(name) GNAC_SAVED_PROFILES_URL(name) ,\
  ".xml"

gchar *saved_profiles_dir;

typedef struct
{
  void* doc;
  void* xpath_context;
}
XMLDoc;

typedef struct
{
  gchar *value;
  gchar *name;
}
Value;

typedef struct
{
  GList *values;
  GList *names;
  gchar *default_value;
  gchar *variable_name;
}
ComboValues;

typedef struct
{
  gdouble min;
  gdouble max;
  gdouble step;
  gdouble default_value;
  gchar *variable_name;
}
SliderValues;

typedef struct
{
  gboolean default_value;
  gchar *variable_name;
} CheckValues;

XMLDoc *
gnac_profiles_xml_engine_load_doc_xpath(const gchar *filename);

gchar *
gnac_profiles_xml_engine_get_format_id( XMLDoc *dx);

gchar *
gnac_profiles_xml_engine_get_text_node( XMLDoc *dx,
                                        const gchar *expr);

GList *
gnac_profiles_xml_engine_get_list_values( XMLDoc *dx,
                                          const gchar *expr);

gint
gnac_profiles_xml_engine_get_combo_values(XMLDoc *dx,
                                          const gchar *expr,
                                          ComboValues *combo_values);
gint
gnac_profiles_xml_engine_get_slider_values(XMLDoc *dx,
                                          const gchar *expr,
                                          SliderValues *slider_values);
gint
gnac_profiles_xml_engine_get_check_values(XMLDoc *dx,
                                          const gchar *expr,
                                          CheckValues *check_values);

void
gnac_profiles_xml_engine_modify_values(XMLDoc *dx, const gchar *expr, ...);

void
gnac_profiles_xml_engine_add_values(XMLDoc *dx, const gchar *expr, va_list ap);

void
gnac_profiles_xml_engine_save_doc(XMLDoc *dx, const gchar *filename);

void
gnac_profiles_xml_engine_free_doc_xpath(XMLDoc *dx);

void
gnac_profiles_xml_engine_free_combo_values(ComboValues *values);

void
gnac_profiles_xml_engine_free_slider_values(SliderValues *values);

void
gnac_profiles_xml_engine_free_check_values(CheckValues *values);

gchar *
gnac_profiles_xml_engine_format_text_to_xml(const gchar *text);

#endif /* GNAC_PROFILES_XML_ENGINE_H */
