/*
 * $Id: gnac-profiles.c 288 2009-03-25 20:30:10Z alexandreroux $
 *
 * This file is part of GNAC - Gnome Audio Converter
 *
 * Copyright (C) 2007 - 2009 Gnac
 *    
 *    - DUPASQUIER  Benoit    <bdupasqu@users.sourceforge.net>
 *    - JOAQUIM     David     <daiji@users.sourceforge.net>
 *    - ROUX        Alexandre <alexandreroux@users.sourceforge.net>
 *
 * GNAC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GNAC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNAC; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include "gnac-profiles.h"
#include "gnac-profiles-manager.h"


static void
gnac_profiles_populate_combo(void);

static AudioProfileGeneric *
gnac_profiles_get_active_profile(void);

static void
gnac_profiles_on_profiles_managed_closed(GtkWidget *widget, gpointer data);

static void
gnac_profiles_on_combo_profile_changed(GtkWidget *widget, gpointer data);

static gchar *
gnac_profiles_get_display_name(void);

static GtkWidget *combo_profile = NULL;

void
gnac_profiles_init(void)
{
  GtkCellRenderer *renderer;
  GtkListStore *model;

  model = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
  combo_profile = gtk_combo_box_new_with_model(GTK_TREE_MODEL(model));
  renderer = gtk_cell_renderer_text_new();
  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo_profile), 
    renderer, FALSE);
  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo_profile), 
    renderer, "text", 0, NULL);
  gtk_combo_box_set_model(GTK_COMBO_BOX(combo_profile),GTK_TREE_MODEL(model));
  g_signal_connect(G_OBJECT(combo_profile), "changed",
    G_CALLBACK(gnac_profiles_on_combo_profile_changed), NULL);

  gnac_profiles_mgr_init((StandardCallBack)
    gnac_profiles_on_profiles_managed_closed);
  gnac_profiles_populate_combo();
}

void
gnac_profiles_destroy(void)
{
  if (combo_profile) {
    gnac_profiles_mgr_destroy();
  }
}

GtkWidget *
gnac_profiles_get_widget(GtkWidget *parent)
{
  if (combo_profile == NULL) {
    gnac_profiles_init();
  }

  gnac_profiles_mgr_set_parent(parent);

  return combo_profile;
}


static gchar *
gnac_profiles_get_display_name(void) {
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *name;

  model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo_profile));
  if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo_profile), &iter)) {
    gtk_tree_model_get(model,&iter,0,(void**)&name,-1);
    return name;
  }

  return NULL;
}

static AudioProfileGeneric *
gnac_profiles_get_active_profile(void)
{
  GtkTreeModel *model;
  GtkTreeIter iter;
  AudioProfileGeneric *profile;

  model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo_profile));
  if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo_profile), &iter)) {
    gtk_tree_model_get(model,&iter,1,(void**)&profile,-1);
    return profile->generic;
  }

  return NULL;
}

const gchar *
gnac_profiles_get_pipeline(void)
{
  AudioProfileGeneric *profile;

  profile = gnac_profiles_get_active_profile();
  if (profile != NULL) {
    return profile->pipeline;
  }

  return NULL;
}

const gchar *
gnac_profiles_get_extension(void)
{
  AudioProfileGeneric *profile;

  profile = gnac_profiles_get_active_profile();
  if (profile != NULL) {
    return profile->extension;
  }

  return NULL;
}

const gchar *
gnac_profiles_get_name(void)
{ 
  AudioProfileGeneric *profile;

  profile = gnac_profiles_get_active_profile();
  if (profile != NULL) {
    return profile->name;
  }

  return NULL;
}

void
gnac_profiles_set_current_profile(const gchar *name) {
  GtkTreeModel *model;
  GtkTreeIter iter;
  AudioProfileGeneric *profile;

  model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo_profile));
  if (gtk_tree_model_get_iter_first(model, &iter)) {
    do {
      gtk_tree_model_get(model,&iter,1,(void**)&profile,-1);
      if (!g_strcmp0(profile->name, name)) {
        gtk_combo_box_set_active_iter(GTK_COMBO_BOX(combo_profile),&iter);
        break;
      }
    } while (gtk_tree_model_iter_next(model, &iter));
  }

}


static void
gnac_profiles_populate_combo(void)
{
  GtkListStore *model;
  GtkTreeIter iter;
  GList *list;
  GList *temp;
  gchar *active_text;
  gboolean set_active;

  model = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(combo_profile)));
 
  active_text = gnac_profiles_get_display_name(); 
  gtk_list_store_clear(model);
  list = gnac_profiles_mgr_get_profiles_list();
  if (list != NULL) {
    gtk_widget_set_sensitive(combo_profile, TRUE);
    set_active = FALSE;
    for (temp = list; temp != NULL; temp = temp->next) {
      gchar *str;
      gchar *formated_name;
      AudioProfileGeneric *profile;
      
      profile = ((AudioProfileGeneric*)temp->data)->generic;
      formated_name = gnac_profiles_properties_filter_text_for_displaying(
        profile->name, MAX_NAME_DISPLAY_SIZE);
      str = g_strconcat(formated_name," (.", profile->extension, ")", NULL);
      gtk_list_store_append(model, &iter);
      gtk_list_store_set(model, &iter, 0, str, 1, profile, -1);
      if (active_text != NULL) {
        if (!g_strcmp0(str,active_text)) {
          gtk_combo_box_set_active_iter(GTK_COMBO_BOX(combo_profile),&iter);
          g_free(active_text);
          active_text = NULL;
          set_active = TRUE;
        }
      }
      g_free(str);
      g_free(formated_name);
    }

    if (!set_active) {
      if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter)) {
        gtk_combo_box_set_active_iter(GTK_COMBO_BOX(combo_profile),&iter);
      }
    }
  
    g_list_free(list);
  } else {
    gtk_widget_set_sensitive(combo_profile, FALSE);
  }
  if (active_text != NULL) g_free(active_text);
}

void
gnac_profiles_on_edit(void)
{
  gnac_profiles_mgr_show();
}

static void
gnac_profiles_on_combo_profile_changed(GtkWidget *widget, gpointer data)
{
  AudioProfileGeneric *profile;
  gchar *description;
  gchar *description_formated;

  profile = gnac_profiles_get_active_profile();
  if (profile != NULL) {
    if ((profile->description != NULL) && (g_strcmp0(profile->description,""))){
      description_formated = gnac_profiles_properties_filter_text_for_displaying(
        profile->description, MAX_DESCR_DISPLAY_SIZE);
      description = gnac_profiles_xml_engine_format_text_to_xml(description_formated); 
      g_free(description_formated);
      description_formated = description;
      description = g_strconcat("<i>",description_formated,"</i>",
        NULL);
      g_free(description_formated);
    } else {
      description = g_strdup("No description");
    }
     
    gtk_widget_set_tooltip_markup(widget,description);
    g_free(description);
  }
}

static void
gnac_profiles_on_profiles_managed_closed(GtkWidget *widget, gpointer data)
{
  gnac_profiles_populate_combo();
}
